/**
 *
 * SIROCCO
 * Copyright (C) 2011 France Telecom
 * Contact: sirocco@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  $Id: DataPushListener.as 760 2011-12-15 22:28:28Z dangtran $
 *
 */

package org.ow2.sirocco.cloudmanager.business
{
    
    import com.adobe.cairngorm.control.CairngormEvent;
    
    import flash.utils.*;
    
    import mx.collections.ArrayCollection;
    import mx.messaging.events.*;
    import mx.messaging.messages.AsyncMessage;
    
    import org.granite.gravity.Consumer;
    import org.ow2.sirocco.cloudmanager.event.GetAllVMImagesEvent;
    import org.ow2.sirocco.cloudmanager.event.dataPush.DataCenterResourceUsageEvent;
    import org.ow2.sirocco.cloudmanager.event.dataPush.DelVMImageEvent;
    import org.ow2.sirocco.cloudmanager.event.dataPush.DelVirtualMachineEvent;
    import org.ow2.sirocco.cloudmanager.event.dataPush.NewEventEvent;
    import org.ow2.sirocco.cloudmanager.event.dataPush.NewVMImageEvent;
    import org.ow2.sirocco.cloudmanager.event.dataPush.NewVirtualMachineEvent;
    import org.ow2.sirocco.cloudmanager.event.dataPush.NewVolumeEvent;
    import org.ow2.sirocco.cloudmanager.event.dataPush.VMImageUpdateEvent;
    import org.ow2.sirocco.cloudmanager.event.dataPush.VMMigrationEvent;
    import org.ow2.sirocco.cloudmanager.event.dataPush.VirtualMachineStatusChangeEvent;
    import org.ow2.sirocco.cloudmanager.event.dataPush.VolumeUpdateEvent;
    import org.ow2.sirocco.cloudmanager.model.CloudModelLocator;
    import org.ow2.sirocco.cloudmanager.utils.ArrayCollectionUtils;
    import org.ow2.sirocco.cloudmanager.utils.logger.Logger;
    import org.ow2.sirocco.cloudmanager.vo.VMImageVO;
    import org.ow2.sirocco.cloudmanager.vo.VirtualMachineVO;
    import org.ow2.sirocco.cloudmanager.vo.VolumeVO;
    
    
    public class DataPushListener
    {
        
        private static var dataPushListener : DataPushListener = null;
        
        private var consumer : Consumer = null;
        
        private var modelLocator : CloudModelLocator = CloudModelLocator.getInstance();
        
        
        
        public static function getInstance() : DataPushListener
        {
            if (dataPushListener == null)
            {
                dataPushListener = new DataPushListener();
            }
            return dataPushListener;
        }
        
        
        
        public function DataPushListener()
        {
            super();
            
            if (DataPushListener.dataPushListener != null)
            {
                throw new Error("Only one DataPushListener instance should be instantiated");
            }
        }
        
        
        
        public function connect() : void
        {
            var modelLocator : CloudModelLocator = CloudModelLocator.getInstance();
            
            consumer = new Consumer();
            consumer.destination = "gravity";
            consumer.topic = "CloudAdmin";
            
            if (!this.modelLocator.isAdmin)
            {
                consumer.selector = "projectId = '" + this.modelLocator.projectId + "'";
            }
            consumer.subscribe();
            consumer.addEventListener(MessageEvent.MESSAGE, messageHandler);
        }
        
        
        
        public function disconnect() : void
        {
            if(consumer!=null) 
            {
                consumer.removeEventListener(MessageEvent.MESSAGE, messageHandler);
                consumer.disconnect();
                consumer = null;
            }
        }
        
        
        
        private function messageHandler(event : MessageEvent) : void
        {
            
            var msg : AsyncMessage = event.message as AsyncMessage;
            var vm : VirtualMachineVO;
            var cEvent : CairngormEvent;
            
            if (msg.body is NewVirtualMachineEvent)
            {
                var newVirtualServerEvent : NewVirtualMachineEvent = msg.body as NewVirtualMachineEvent;
                Logger.debug("Received message : ProjectId = '" + msg.headers.projectId + "'; Event = " + getQualifiedClassName(newVirtualServerEvent) + "; Server = " + newVirtualServerEvent.server);
                
                this.modelLocator.virtualMachinesVoList.addItem(newVirtualServerEvent.server);
                
                if (this.modelLocator.projectId == newVirtualServerEvent.server.projectId)
                    this.modelLocator.virtualMachinesVoByProjectList.addItem(newVirtualServerEvent.server);
                
                // Update the virtualMachineImage to set the image name in VirtualMachineVO
                cEvent = new GetAllVMImagesEvent();
                cEvent.dispatch();
            }
            else if (msg.body is DelVirtualMachineEvent)
            {
                var delVirtualServerEvent : DelVirtualMachineEvent = DelVirtualMachineEvent(msg.body);
                Logger.debug("Received message : ProjectId = '" + msg.headers.projectId + "'; Event = " + getQualifiedClassName(delVirtualServerEvent) + "; VmId = " + delVirtualServerEvent.vmId);
                
                if (this.modelLocator.virtualMachinesVoList.length != 0)
                {
                    ArrayCollectionUtils.removeItemInCollection(this.modelLocator.virtualMachinesVoList, "id", delVirtualServerEvent.vmId);
                }
                
                if (this.modelLocator.virtualMachinesVoByProjectList.length != 0)
                {
                    ArrayCollectionUtils.removeItemInCollection(this.modelLocator.virtualMachinesVoByProjectList, "id", delVirtualServerEvent.vmId);
                }
                
                // Remove the VM from its host in the DataCenter hierarchy
                removeVirtualMachineFromItsHost(delVirtualServerEvent.vmId, delVirtualServerEvent.hostId);
            }
            else if (msg.body is VirtualMachineStatusChangeEvent)
            {
                var virtualServerStatusChangeEvent : VirtualMachineStatusChangeEvent = msg.body as VirtualMachineStatusChangeEvent;
                Logger.debug("Received message : ProjectId = '" + msg.headers.projectId + "'; Event = " + getQualifiedClassName(virtualServerStatusChangeEvent) + "; Server = " + virtualServerStatusChangeEvent.server);
                
                // Search the VM in the model's VM list
                if (this.modelLocator.virtualMachinesVoByProjectList.length != 0)
                {
                    // Update the VM properties
                    vm = ArrayCollectionUtils.findInCollection(this.modelLocator.virtualMachinesVoByProjectList, "id", virtualServerStatusChangeEvent.server.id) as VirtualMachineVO;
                    
                    if (vm != null)
                    {
                        vm.status = virtualServerStatusChangeEvent.server.status;
                        vm.privateIp = virtualServerStatusChangeEvent.server.privateIp;
                        vm.privateDnsName = virtualServerStatusChangeEvent.server.privateDnsName;
                        vm.publicIp = virtualServerStatusChangeEvent.server.publicIp;
                        vm.publicDnsName = virtualServerStatusChangeEvent.server.publicDnsName;
                        vm.creationDate = virtualServerStatusChangeEvent.server.creationDate;
                    }
                }
                
                if (this.modelLocator.virtualMachinesVoList.length != 0)
                {
                    // Update the VM properties
                    vm = ArrayCollectionUtils.findInCollection(this.modelLocator.virtualMachinesVoList, "id", virtualServerStatusChangeEvent.server.id) as VirtualMachineVO;
                    
                    if (vm != null)
                    {
                        vm.status = virtualServerStatusChangeEvent.server.status;
                        vm.privateIp = virtualServerStatusChangeEvent.server.privateIp;
                        vm.privateDnsName = virtualServerStatusChangeEvent.server.privateDnsName;
                        vm.publicIp = virtualServerStatusChangeEvent.server.publicIp;
                        vm.publicDnsName = virtualServerStatusChangeEvent.server.publicDnsName;
                        vm.creationDate = virtualServerStatusChangeEvent.server.creationDate;
                    }
                }
                
                // If it's a new VM, add it to its host in the DataCenter hierarchy
                addNewVirtualMachineToItsHost(vm, virtualServerStatusChangeEvent.server.hostId);
            }
            else if (msg.body is DelVMImageEvent)
            {
                var delVMImageEvent : DelVMImageEvent = msg.body as DelVMImageEvent;
                Logger.debug("Received message : ProjectId = '" + msg.headers.projectId + "'; Event = " + getQualifiedClassName(delVMImageEvent) + "; ImageId = " + delVMImageEvent.vmiId);
                
                var image : VMImageVO;
                
                if (this.modelLocator.vmImagesVoList.length != 0)
                {
                    image = ArrayCollectionUtils.findInCollection(this.modelLocator.vmImagesVoList, "id", delVMImageEvent.vmiId) as VMImageVO;
                    
                    if (image != null)
                        this.modelLocator.vmImagesVoList.removeItemAt(this.modelLocator.vmImagesVoList.getItemIndex(image));
                }
                
                if (this.modelLocator.vmImagesVoByProjectList.length != 0)
                {
                    image = ArrayCollectionUtils.findInCollection(this.modelLocator.vmImagesVoByProjectList, "id", delVMImageEvent.vmiId) as VMImageVO;
                    
                    if (image != null)
                        this.modelLocator.vmImagesVoByProjectList.removeItemAt(this.modelLocator.vmImagesVoByProjectList.getItemIndex(image));
                }
            }
            else if (msg.body is NewVMImageEvent)
            {
                var newVMImageEvent : NewVMImageEvent = msg.body as NewVMImageEvent;
                Logger.debug("Received message : ProjectId = '" + msg.headers.projectId + "'; Event = " + getQualifiedClassName(newVMImageEvent) + "; Image = " + newVMImageEvent.image);
                
                this.modelLocator.vmImagesVoByProjectList.addItem(newVMImageEvent.image);
                this.modelLocator.vmImagesVoList.addItem(newVMImageEvent.image);
            }
            else if (msg.body is VMImageUpdateEvent)
            {
                var vMImageUpdateEvent : VMImageUpdateEvent = msg.body as VMImageUpdateEvent;
                Logger.debug("Received message : ProjectId = '" + msg.headers.projectId + "'; Event = " + getQualifiedClassName(vMImageUpdateEvent) + "; Image = " + vMImageUpdateEvent.image);
                
                if (msg.headers.projectId == this.modelLocator.projectId && this.modelLocator.vmImagesVoList.length != 0)
                {
                    image = ArrayCollectionUtils.findInCollection(this.modelLocator.vmImagesVoList, "id", vMImageUpdateEvent.image.id) as VMImageVO;
                    
                    if (image != null)
                        image.initFrom(vMImageUpdateEvent.image);
                }
                
                if (this.modelLocator.vmImagesVoByProjectList.length != 0)
                {
                    image = ArrayCollectionUtils.findInCollection(this.modelLocator.vmImagesVoByProjectList, "id", vMImageUpdateEvent.image.id) as VMImageVO;
                    
                    if (image != null)
                        image.initFrom(vMImageUpdateEvent.image);
                }
            }
            else if (msg.body is DataCenterResourceUsageEvent)
            {
                var usageEvent : DataCenterResourceUsageEvent = msg.body as DataCenterResourceUsageEvent;
                this.modelLocator.updateDataCenterResourceUsage(usageEvent.rootDomain);
                this.modelLocator.dataCenterResourceUsageUpdateWatcher++
            }
            else if (msg.body is NewEventEvent)
            {
                var e : NewEventEvent = msg.body as NewEventEvent;
                Logger.debug("Received message : NewEventEvent " + e.event);
                
                if (this.modelLocator.currentPageNumber == 0)
                {
                    this.modelLocator.eventVoList.addItemAt(e.event, 0);
                    
                    if (this.modelLocator.eventVoList.length > this.modelLocator.pageSize)
                        this.modelLocator.eventVoList.removeItemAt(this.modelLocator.eventVoList.length - 1);
                }
                this.modelLocator.eventCount++;
            }
            else if (msg.body is VMMigrationEvent)
            {
                var vmMigrationEvent : VMMigrationEvent = msg.body as VMMigrationEvent;
                Logger.debug("Received message : ProjectId = '" + msg.headers.projectId + "'; Event = " + getQualifiedClassName(vmMigrationEvent) + "; vmId = " + vmMigrationEvent.vmId + "; fromHostId = " + vmMigrationEvent.fromHostId + "; toHostId = " + vmMigrationEvent.toHostId);
                
                // UPDATE THE DATACENTER HIERARCHY
                
                var fromHost : Object = CloudModelLocator.getInstance().objResourceTable[vmMigrationEvent.fromHostId];
                var vmVo : VirtualMachineVO = null;
                var refreshNeeded : Boolean = false;
                
                if (fromHost != null)
                {
                    vmVo = ArrayCollectionUtils.removeItemInCollection(fromHost.treenode.children, "id", vmMigrationEvent.vmId) as VirtualMachineVO;
                    refreshNeeded = true;
                }
                var toHost : Object = CloudModelLocator.getInstance().objResourceTable[vmMigrationEvent.toHostId];
                
                if (toHost != null && vmVo != null)
                {
                    ArrayCollection(toHost.treenode.children).addItem(vmVo);
                    refreshNeeded = true;
                }
                
                if (refreshNeeded)
                    this.modelLocator.fullDataCenterResourceTreeUpdateWatcher++;
            } else if(msg.body is NewVolumeEvent) 
            {
                var newVolumeEvent:NewVolumeEvent=msg.body as NewVolumeEvent;
                Logger.debug("Received message : ProjectId = '" + msg.headers.projectId + "'; Event = " + getQualifiedClassName(newVolumeEvent) + "; volume = " + newVolumeEvent.volume);
                this.modelLocator.volumeVoByProjectList.addItem(newVolumeEvent.volume);
                modelLocator.volumeVoList.addItem(newVolumeEvent.volume);
            } else if (msg.body is VolumeUpdateEvent)
            {
                var volumeUpdateEvent : VolumeUpdateEvent = msg.body as VolumeUpdateEvent;
                var volume:VolumeVO;
                Logger.debug("Received message : ProjectId = '" + msg.headers.projectId + "'; Event = " + getQualifiedClassName(volumeUpdateEvent) + "; Volume = " + volumeUpdateEvent.volume);
                
                if (msg.headers.projectId == this.modelLocator.projectId && this.modelLocator.volumeVoList.length != 0)
                {
                    var index:int= ArrayCollectionUtils.findIndex(this.modelLocator.volumeVoList,  volumeUpdateEvent.volume.volumeId, "volumeId");
                    if(index!=-1) 
                    {
                        volume=this.modelLocator.volumeVoList[index];
                        volume.initFrom(volumeUpdateEvent.volume);
                        this.modelLocator.volumeVoList.setItemAt(volume,index);
                    }
                }
                
                if (this.modelLocator.volumeVoByProjectList.length != 0)
                {
                    index= ArrayCollectionUtils.findIndex(this.modelLocator.volumeVoByProjectList,  volumeUpdateEvent.volume.volumeId, "volumeId");
                    if(index!=-1) 
                    {
                        Logger.debug("Update Volume :"+index);
                        volume=this.modelLocator.volumeVoByProjectList[index];
                        volume.initFrom(volumeUpdateEvent.volume);
                        this.modelLocator.volumeVoByProjectList.setItemAt(volume,index);
                    }
                }
            }
        }
        
        
        
        /**
         * Add the VM to its host in the DataCenter hierarchy (if it's a new VM)
         * @param vm The Virtual Machine to add.
         * @param hostId The ID of the Host that hosts the VM.
         */
        private function addNewVirtualMachineToItsHost(vm : VirtualMachineVO, hostId : String) : void
        {
            var fullDataCenterResourceTree : Object = CloudModelLocator.getInstance().objFullDataCenterResourceTree;
            
            if (fullDataCenterResourceTree != null)
            {
                var host : Object = CloudModelLocator.getInstance().objResourceTable[hostId];
                
                if (host != null && vm != null)
                {
                    if (ArrayCollectionUtils.findInCollection(host.treenode.children, "id", vm.id) == null)
                    {
                        ArrayCollection(host.treenode.children).addItem(vm);
                        CloudModelLocator.getInstance().fullDataCenterResourceTreeUpdateWatcher++;
                    }
                }
            }
        }
        
        
        
        /**
         * Remove the VM from its host in the DataCenter hierarchy
         * @param id ID of the Virtual Machine to remove.
         */
        private function removeVirtualMachineFromItsHost(id : String, hostId : String) : void
        {
            var fullDataCenterResourceTree : Object = CloudModelLocator.getInstance().objFullDataCenterResourceTree;
            
            if (fullDataCenterResourceTree != null)
            {
                var host : Object = CloudModelLocator.getInstance().objResourceTable[hostId];
                
                if (host != null)
                {
                    if (ArrayCollectionUtils.removeItemInCollection(host.treenode.children, "id", id) != null)
                        CloudModelLocator.getInstance().fullDataCenterResourceTreeUpdateWatcher++;
                }
            }
        }
    
    }
}


