/**
 *
 * SIROCCO
 * Copyright (C) 2011 France Telecom
 * Contact: sirocco@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  $Id: GetDataCenterResourceTreeCommand.as 760 2011-12-15 22:28:28Z dangtran $
 *
 */

package org.ow2.sirocco.cloudmanager.command
{
    
    import com.adobe.cairngorm.commands.*;
    import com.adobe.cairngorm.control.*;
    
    import mx.collections.ArrayCollection;
    import mx.collections.Sort;
    import mx.collections.SortField;
    import mx.managers.CursorManager;
    import mx.rpc.IResponder;
    import mx.rpc.events.ResultEvent;
    
    import org.ow2.sirocco.cloudmanager.business.*;
    import org.ow2.sirocco.cloudmanager.event.*;
    import org.ow2.sirocco.cloudmanager.model.*;
    import org.ow2.sirocco.cloudmanager.vo.*;
    
    public class GetDataCenterResourceTreeCommand extends SequenceCommand implements IResponder
    {
        
        
        override public function execute(event:CairngormEvent):void
        {
            // Set the "busy" cursor
            CursorManager.setBusyCursor();
            
            var getDataCenterResourceTreeEvent:GetDataCenterResourceTreeEvent=event as GetDataCenterResourceTreeEvent;
            
            var delegate:GetDataCenterResourceTreeDelegate=CloudModelLocator.getInstance().getDataCenterResourceTreeDelegate;
            delegate.responder=this;
            
            delegate.getDataCenterResourceTree();
        }
        
        private function populateDomain(model:CloudModelLocator, subDomainVo:DomainVO, domain:Object):Object
        {
            var sortByDomainName:Sort=new Sort();
            sortByDomainName.fields=[new SortField("name")];
            subDomainVo.subDomains.sort=sortByDomainName;
            subDomainVo.subDomains.refresh();
            
            
            var objSubDomain:Object={name: subDomainVo.name, type: "domain", vo: subDomainVo, children: new ArrayCollection()};
            model.objResourceTable[subDomainVo.id]={"container": subDomainVo, "treenode": objSubDomain};
            if (domain != null)
                ArrayCollection(domain.children).addItem(objSubDomain);
            
            // Sort server pools by name
            var sortByServerPoolName:Sort=new Sort();
            sortByServerPoolName.fields=[new SortField("name")];
            subDomainVo.serverPools.sort=sortByServerPoolName;
            subDomainVo.serverPools.refresh();
            for each (var serverPool:ServerPoolVO in subDomainVo.serverPools)
            {
                var objServerPool:Object={name: serverPool.name, type: "serverPool", vo: serverPool, children: new ArrayCollection()};
                model.objResourceTable[serverPool.id]={"container": serverPool, "treenode": objServerPool};
                ArrayCollection(objSubDomain.children).addItem(objServerPool);
                
                // Sort hosts by name
                var sortByHostName:Sort=new Sort();
                sortByHostName.fields=[new SortField("hostName")];
                serverPool.hosts.sort=sortByHostName;
                serverPool.hosts.refresh();
                for each (var host:HostVO in serverPool.hosts)
                {
                    var objHost:Object={name: host.hostName, type: "host", vo: host, children: new ArrayCollection()};
                    model.objResourceTable[host.id]={"container": host, "treenode": objHost};
                    var vm:VirtualMachineVO=new VirtualMachineVO();
                    vm.name=model.resourceManager.common_loading;
                    ArrayCollection(objHost.children).addItem(vm);
                    ArrayCollection(objServerPool.children).addItem(objHost);
                }
            }
            for each (var subSubDomainVo:DomainVO in subDomainVo.subDomains)
            {
                populateDomain(model, subSubDomainVo, objSubDomain);
            }
            return objSubDomain;
        }
        
        public function result(event:Object):void
        {
            CursorManager.removeAllCursors();
            
            var resultEvent:ResultEvent=event as ResultEvent;
            var domainVo:DomainVO;
            
            if(resultEvent.result!=null) 
            {
                domainVo=resultEvent.result as DomainVO;
            } else 
            {
                domainVo=new DomainVO();
                domainVo.name="root";
                domainVo.subDomains=new ArrayCollection();
                domainVo.serverPools=new ArrayCollection();
            }
            
            // Gets the Model
            var model:CloudModelLocator=CloudModelLocator.getInstance();
            
            // Put the domain in the model
            model.dataCenterResourceTree=domainVo;
            model.objResourceTable=new Object();
            
            //TODO: Build the VM hierarchy for the "DataCenter" mode in Inventory
            // (ServerPools and SubDomains are null at the moment).
            //var virtualMachineDataCenterList:ArrayCollection = new ArrayCollection();
//			var virtualMachineDataCenter:VirtualMachineDataCenterVO;
//			var dataCenterHierarchy:Object=new Object();
//			dataCenterHierarchy.name=domain.name;
//			dataCenterHierarchy.vo=domain;
//			dataCenterHierarchy.type="domain";
//			dataCenterHierarchy.children=new ArrayCollection();
//
//			model.resourceContainers[domain.id]=domain;
//
//			// Sort domains by name
//			var sortByDomainName:Sort=new Sort();
//			sortByDomainName.fields=[new SortField("name")];
//			domain.subDomains.sort=sortByDomainName;
//			domain.subDomains.refresh();
            
            model.objFullDataCenterResourceTree=populateDomain(model, domainVo, null);
            //model.virtualMachineDataCenterList.source = virtualMachineDataCenterList.source;
            model.fullDataCenterResourceTreeUpdateWatcher++;
            model.dataCenterResourceUsageUpdateWatcher++;
        }
        
        /**
        * Manage failure : show the error message.
        */
        public function fault(event:Object):void
        {
            CursorManager.removeAllCursors();
            
            var model:CloudModelLocator=CloudModelLocator.getInstance();
            
            model.onCommandFault(event);
        }
    }
}


