/**
 *
 * SIROCCO
 * Copyright (C) 2011 France Telecom
 * Contact: sirocco@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  $Id: GetVirtualMachinePerfMetricsCommand.as 760 2011-12-15 22:28:28Z dangtran $
 *
 */

package org.ow2.sirocco.cloudmanager.command
{

import com.adobe.cairngorm.commands.*;
import com.adobe.cairngorm.control.*;
import org.ow2.sirocco.cloudmanager.business.*;
import org.ow2.sirocco.cloudmanager.event.*;
import org.ow2.sirocco.cloudmanager.model.*;

import mx.rpc.IResponder;
import mx.rpc.events.ResultEvent;


public class GetVirtualMachinePerfMetricsCommand extends SequenceCommand implements IResponder
{
    
    private var vmId : String;
    
    private var metricId : String;
    
    private var startTime : Date;
    
    private var endTime : Date;
    
    private var stacked : Boolean;
    
    
    
    public function GetVirtualMachinePerfMetricsCommand()
    {
    }
    
    
    
    override public function execute(event : CairngormEvent) : void
    {
        // Set the "busy" cursor
        //CursorManager.setBusyCursor();
        
        var model : CloudModelLocator = CloudModelLocator.getInstance();
        
        var cEvent : GetVirtualMachinePerfMetricsEvent = event as GetVirtualMachinePerfMetricsEvent;
        vmId = cEvent.vmId;
        metricId = cEvent.metricId;
        startTime = cEvent.startTime;
        endTime = cEvent.endTime;
        stacked = cEvent.stacked;
        
        var delegate : GetVirtualMachinePerfMetricsDelegate = CloudModelLocator.getInstance().getVirtualMachinePerfMetricsDelegate;
        delegate.responder = this;
        
        delegate.getVirtualMachinePerfMetrics(vmId, metricId, startTime, endTime);
    }
    
    
    
    public function result(event : Object) : void
    {
        //CursorManager.removeAllCursors();
        
        var resultEvent : ResultEvent = event as ResultEvent;
        var array : Array = resultEvent.result as Array;
        
        /// Transform the array to another array more convenient to feed Flex Charts.
        
        var perfMetricsComplete : Array = new Array();
        var nbItems : uint = array.length;
        
        if (nbItems > 0)
        {
            var components : Array;
            var isComposite : Boolean;
            
            // XXX no composite perf metrics received for the time being
            isComposite=false;
            
            if (isComposite)
            {
                // Get all components
                // (we need to search in all the data because some components may appear/disappear according to the time)
                components = new Array();
                
                for (var iItem1 : int = 0; iItem1 < nbItems; iItem1++)
                {
                    for (var iComponent1 : int; iComponent1 < array[iItem1].components.length; iComponent1++)
                    {
                        if (components.indexOf(array[iItem1].components[iComponent1]) < 0)
                        {
                            components.push(array[iItem1].components[iComponent1]);
                        }
                    }
                }
            }
            else
            {
                isComposite = false;
                components = ["values"];
            }
            
            for (var iComponent : int; iComponent < components.length; iComponent++)
            {
                var perfMetrics : Array = new Array();
                
                for (var iItem2 : int = 0; iItem2 < nbItems; iItem2++)
                {
                    var iActualComponent : int = -1;
                    
                    if (isComposite)
                    {
                        iActualComponent = array[iItem2].components.indexOf(components[iComponent]);
                        perfMetrics.push({date: (array[iItem2].timeStamp as Date), value: ((iActualComponent >= 0) ? array[iItem2].values[iActualComponent] : 0)});
                    }
                    else
                    {
                        perfMetrics.push({date: (array[iItem2].timeStamp as Date), value: array[iItem2].value});
                    }
                }
                perfMetricsComplete.push({title: components[iComponent], series: perfMetrics});
            }
            
            // Make STACKED perf metrics (additioning each series)
            if (stacked && (perfMetricsComplete.length > 0))
            {
                for (var iItem3 : int = 0; iItem3 < nbItems; iItem3++)
                {
                    var addedValue : Number = 0;
                    
                    for (var iComponent3 : int = 0; iComponent3 < components.length; iComponent3++)
                    {
                        addedValue += perfMetricsComplete[iComponent3].series[iItem3].value;
                        perfMetricsComplete[iComponent3].series[iItem3].value = addedValue;
                    }
                }
            }
        }
        
        // Gets the Model
        var model : CloudModelLocator = CloudModelLocator.getInstance();
        
        //model.perfMetrics.source = array;
        model.perfMetrics.source = perfMetricsComplete;
    }
    
    
    
    /**
     * Manage failure : show the error message.
     */
    public function fault(event : Object) : void
    {
        //CursorManager.removeAllCursors();
        
        var model : CloudModelLocator = CloudModelLocator.getInstance();
        
        model.onCommandFault(event);
    }
}
}


