/**
 *
 * SIROCCO
 * Copyright (C) 2011 France Telecom
 * Contact: sirocco@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  $Id: OOCModelLocator.as 754 2011-12-15 18:20:47Z dangtran $
 *
 */

package org.ow2.sirocco.cloudmanager.model
{
    
    import com.adobe.cairngorm.model.IModelLocator;
    
    import flash.events.Event;
    import flash.net.URLRequest;
    
    import mx.collections.ArrayCollection;
    import mx.controls.Alert;
    import mx.core.Application;
    import mx.rpc.events.FaultEvent;
    
    import org.ow2.sirocco.cloudmanager.business.*;
    import org.ow2.sirocco.cloudmanager.event.*;
    import org.ow2.sirocco.cloudmanager.utils.PopUpUtils;
    import org.ow2.sirocco.cloudmanager.utils.internationalization.SiroccoResourceManager;
    import org.ow2.sirocco.cloudmanager.vo.*;
    
    
    /**
     * Class representing the "Model" part of the MVC (Model-View-Controller) Design Pattern.
     * The ModelLocator is a centralized repository for all the data that is needed accross the application.
     * @author ARLERAY
     */
    public class CloudModelLocator implements IModelLocator
    {
        // Single instance of our ModelLocator
        private static var instance : CloudModelLocator = null;
        
        
        
        /**
         * Constructor of the CloudModelLocator class.
         * <b>DO NOT USE THIS CONSTRUCTOR!
         * Instead, use <code>CloudModelLocator.getInstance()</code></b> (Singleton Design Pattern)
         *
         * <p>Note: we use the Singleton Design Pattern for this class, but ActionScript 3
         * does not allow private constructors. So, to make the constructor unusable,
         * we use an inner class (SingletonEnforcer) as parameter.</p>
         * @param enforcer Inner class parameter, used to make the constructor unusable.
         */
        public function CloudModelLocator(enforcer : SingletonEnforcer)
        {
            if (enforcer == null)
            {
                throw new Error("CloudModelLocator constructor forbidden! Instead, use CloudModelLocator.getInstance().");
            }
        }
        
        
        
        /**
         * Returns a unique instance of CloudModelLocator (Singleton Design Pattern).
         * @return The unique instance of CloudModelLocator.
         */
        public static function getInstance() : CloudModelLocator
        {
            if (instance == null)
            {
                instance = new CloudModelLocator(new SingletonEnforcer());
            }
            
            return instance;
        }
        
        // INVENTORY VARIABLES
        
        [Bindable]
        public var cpuLoadArray : Array = new Array();
        
        [Bindable]
        public var cpuLoadArrayUpdateWatcher : uint;
        
        [Bindable]
        public var memoryUsedArray : Array = new Array();
        
        [Bindable]
        public var memoryUsedArrayUpdateWatcher : uint;
        
        [Bindable]
        public var dataCenterResourceTree : DomainVO = null;
        
        public var objResourceTable : Object = null;
        
        [Bindable]
        public var objFullDataCenterResourceTree : Object = null;
        
        // Used to inform that the DataCenter hierarchy has been updated
        // (in order to update the associated UI)
        [Bindable]
        public var fullDataCenterResourceTreeUpdateWatcher : uint;
        
        // Used to inform that the ResourceUsage view has to be updated
        [Bindable]
        public var dataCenterResourceUsageUpdateWatcher : uint;
        
        // Used to inform that a VM migration has failed, so the VM Placement view has to be updated
        [Bindable]
        public var vmMigrationFailedWatcher : uint;
        
        // USER MANAGEMENT VARIABLES
        
        [Bindable]
        public var loadingUserVoList : Boolean;
        
        [Bindable]
        [ArrayElementType(UserVO)]
        public var allUsersVoList : ArrayCollection = new ArrayCollection();
        
        [Bindable]
        public var username : String;
        
        [Bindable]
        public var projectId : String;
        
        [Bindable]
        public var projectName : String;
        
        [Bindable]
        public var userVo : UserVO = null;
        
        [Bindable]
        public var userLdapVo : UserVO = null;
        
        [Bindable]
        public var projectManagementSelectedProjectVo : ProjectVO = null;
        
        [Bindable]
        public var projectResourceQuotaVo : ResourceQuotaVO = null;
        
        [Bindable]
        public var projectResourceInfoVo : ResourceConsumption = null;
        
        [Bindable]
        [ArrayElementType(VirtualMachineVO)]
        public var virtualMachinesVoList : ArrayCollection = new ArrayCollection();
        
        public var vmSizeName : String = null;
        
        [Bindable]
        [ArrayElementType(VMSizeVO)]
        public var vmSizesVoList : ArrayCollection = new ArrayCollection();
        
        // VARIABLES
        
        [Bindable]
        [ArrayElementType(VirtualMachineVO)]
        public var virtualMachinesVoByProjectList : ArrayCollection = new ArrayCollection();
        
        [Bindable]
        public var loadingVirtualMachinesVoByProjectList : Boolean;
        
        [Bindable]
        [ArrayElementType(VMImageVO)]
        public var vmImagesVoList : ArrayCollection = new ArrayCollection();
        
        [Bindable]
        [ArrayElementType(VolumeVO)]
        public var volumeVoByProjectList : ArrayCollection = new ArrayCollection();
        
        [Bindable]
        [ArrayElementType(VolumeVO)]
        public var volumeVoList : ArrayCollection = new ArrayCollection();
        
        [Bindable]
        [ArrayElementType(ProjectVO)]
        public var projectsVoList : ArrayCollection = new ArrayCollection();
        
        [Bindable]
        public var loadingVMImagesVoList : Boolean;
        
        [Bindable]
        public var loadingAllVMImagesVoList : Boolean;
        
        [Bindable]
        public var loadingAllProjectsVoList : Boolean;
        
        [Bindable]
        public var cloudProviderLocations : Object=new Object();
        
        [Bindable]
        [ArrayElementType(VMImageVO)]
        public var vmImagesVoByProjectList : ArrayCollection = new ArrayCollection();
        
        [Bindable]
        [ArrayElementType(ProjectVO)]
        public var projectsVoByUserList : ArrayCollection = new ArrayCollection();
        
        [Bindable]
        public var loadingVMImagesVoByProjectList : Boolean;
        
        [Bindable]
        public var loadingVolumes : Boolean;
        
        [Bindable]
        public var loadingProjectsVoByUserList : Boolean;
        
        [Bindable]
        public var resourceManager : SiroccoResourceManager = SiroccoResourceManager.getInstance();
        
        [Bindable]
        public var _workflowState : uint = CloudModelLocator.LOGIN_SCREEN;
        
        
        
        [Bindable]
        public function get workflowState() : uint
        {
            return this._workflowState;
        }
        
        
        
        public function set workflowState(workflowState : uint) : void
        {
            this._workflowState = workflowState;
        }
        
        [Bindable]
        public var isLogged : Boolean;
        
        [Bindable]
        public var isAdmin : Boolean;
        
        [Bindable]
        public var sessionTokenId : String;
        
        [Bindable]
        public var loadingVirtualMachines : Boolean;
        
        [Bindable]
        public var loadingVirtualMachineByVmId : Boolean;
        
        //VirtualMachineView
        [Bindable]
        public var virtualMachineVo : VirtualMachineVO = null;
        
        [Bindable]
        public var virtualMachineExtendedInfo : VirtualMachineExtendedInfo = null;
        
        [Bindable]
        public var loadingVirtualMachineExtendedInfo : Boolean;
        
        [Bindable]
        public var loadingVirtualMachineConsole : Boolean;
        
        [Bindable]
        public var hostPerfMetrics : ArrayCollection = new ArrayCollection();
        
        [Bindable]
        public var hostPerfMetricInfos : ArrayCollection = new ArrayCollection();
        
        [Bindable]
        public var virtualMachinePerfMetricInfos : ArrayCollection = new ArrayCollection();
        
        [Bindable]
        public var perfMetrics : ArrayCollection = new ArrayCollection();
        
        public const PERF_REFRESH_PERIOD_MS : int = 5000;
        
        [Bindable]
        public var cloudProviderAccounts : ArrayCollection = new ArrayCollection();
        
        [Bindable]
        public var cloudProviders : ArrayCollection = new ArrayCollection();
        
        [Bindable]
        public var currentCloudProviderId: String;
        
        [Bindable]
        public var currentCloudProviderLocations: ArrayCollection = new ArrayCollection();
        
        [Bindable]
        public var currentCloudProviderLocation:CloudProviderLocation;
        
        [Bindable]
        [ArrayElementType(EventVO)]
        public var eventVoList : ArrayCollection = new ArrayCollection();
        
        [Bindable]
        public var eventCount : int;
        
        [Bindable]
        public var currentPageNumber : int = 0;
        
        [Bindable]
        public var pageSize : int;
        
        private var _comboProjectVoListSelectedIndex : int;
        
        
        
        [Bindable]
        public function get comboProjectVoListSelectedIndex() : int
        {
            return _comboProjectVoListSelectedIndex;
        }
        
        
        
        public function set comboProjectVoListSelectedIndex(value : int) : void
        {
            _comboProjectVoListSelectedIndex = value;
        }
        
        // Language management
        [Bindable]
        public var languages : Array = ["en_US", "fr_FR"];
        
        private var _selectedLanguageIndex : uint = 0;
        
        
        
        [Bindable]
        public function get selectedLanguageIndex() : uint
        {
            return this._selectedLanguageIndex;
        }
        
        
        
        public function set selectedLanguageIndex(index : uint) : void
        {
            this._selectedLanguageIndex = index;
            this.resourceManager.locale = languages[this._selectedLanguageIndex];
        }
        
        // DB PARAMETERS VARIABLES
        [Bindable]
        public var TARGET_AUDIENCE : String;
        
        public const UI_BRANDING : String = CONFIG::Branding;
        
        // CONSTANTS
        
        public static const LOGIN_SCREEN : uint = 0;
        
        // USER VIEW CONSTANTS
        public static const USER_VIRTUAL_MACHINES : uint = 1;
        
        public static const USER_VM_IMAGES : uint = 2;
        
        public static const USER_VOLUMES : uint = 3;
        
        public static const USER_PROJECTS : uint = 4;
        
        public static const USER_ACCOUNT : uint = 5;
        
        // ADMIN VIEW CONSTANTS
        public static const INVENTORY_SCREEN : uint = 6;
        
        public static const IMAGE_MANAGEMENT_SCREEN : uint = 7;
        
        public static const VOLUME_MANAGEMENT_SCREEN : uint = 8;
        
        public static const PROJECT_MANAGEMENT_SCREEN : uint = 9;
        
        public static const USER_MANAGEMENT_SCREEN : uint = 10;
        
        // COMMON VIEW CONSTANTS
        public static const EVENT_SCREEN : uint = 11;
        
        
        
        public function updateDataCenterResourceUsage(domainVo : DomainVO) : void
        {
            updateResourceUsage(domainVo, domainVo.id);
            
            for each (var subDomainVo : DomainVO in domainVo.subDomains)
            {
                updateDataCenterResourceUsage(subDomainVo);
            }
            
            for each (var serverPoolVo : ServerPoolVO in domainVo.serverPools)
            {
                updateResourceUsage(serverPoolVo, serverPoolVo.id);
                
                for each (var hostVo : HostVO in serverPoolVo.hosts)
                {
                    updateResourceUsage(hostVo, hostVo.id);
                }
            }
        }
        
        
        
        private function updateResourceUsage(rc : ResourceContainer, id : String) : void
        {
            if (objResourceTable[id] != null)
            {
                ResourceContainer(objResourceTable[id].container).initFrom(rc);
            }
        }
        
        public function getCloudProviderDescription(cloudProviderType:String) :String
        {
            for each (var cloudProvider:CloudProviderVO in cloudProviders)
            {
                if(cloudProvider.cloudProviderType==cloudProviderType) return cloudProvider.description;
            }
            return "?";
        }
        
        
        
        public function reset() : void
        {
            this.isLogged = false;
            this.username = null;
            this.projectId = null;
            this.projectName = null;
            this.cpuLoadArray = new Array();
            this.memoryUsedArray = new Array();
            this.dataCenterResourceTree = null;
            this.objResourceTable = new Object();
            this.objFullDataCenterResourceTree = null;
            this.allUsersVoList.source = new Array();
            this.userVo = null;
            this.userLdapVo = null;
            this.projectResourceQuotaVo = null;
            this.projectsVoByUserList.source = new Array();
            this.projectsVoList.source = new Array();
            this.projectResourceInfoVo = null;
            this.virtualMachinesVoList.source = new Array();
            this.vmSizeName = null;
            this.vmSizesVoList = new ArrayCollection();
            this.virtualMachinesVoByProjectList.source = new Array();
            this.vmImagesVoList.source = new Array();
            this.volumeVoByProjectList.source = new Array();
            this.cloudProviderLocations=new Object();
            this.vmImagesVoByProjectList.source = new Array();
            this.hostPerfMetrics.source = new Array();
            this.perfMetrics.source = new Array();
            this.sessionTokenId = null;
            this.selectedLanguageIndex = 0;
            this.TARGET_AUDIENCE = null;
            this.resourceManager.locale = "en_US";
        }
        
        /**
         * FAULT / ERROR MANAGEMENT
         **/
        
        private var lastError : FaultEvent = null;
        
        
        
        private function reloadPage(event : Event) : void
        {
            var ref : URLRequest = new URLRequest("javascript:location.reload(true)");
            flash.net.navigateToURL(ref, "_self");
        }
        
        
        
        public function onCommandFault(event : Object) : void
        {
            var faultEvent : FaultEvent = FaultEvent(event);
            
            if ((this.lastError == null) || (faultEvent.fault.faultString != "") || (faultEvent.fault.faultCode != this.lastError.fault.faultCode))
            {
                this.lastError = faultEvent;
                
                if (faultEvent.fault.faultDetail.indexOf("InvalidSessionException:") != -1)
                {
                    PopUpUtils.closeAllPopups();
                    reset();
                    workflowState = CloudModelLocator.LOGIN_SCREEN;
                    Alert.show("Your session has expired. Please log in again", "Info", mx.controls.Alert.OK, null, reloadPage);
                }
                else if (faultEvent.fault.faultDetail.indexOf("InvalidUsernameException:") != -1)
                {
                    Alert.show(this.resourceManager.errorMessage_invalidUsernameExceptionMessage, this.resourceManager.common_error);
                }
                else if (faultEvent.fault.faultDetail.indexOf("MachineInUseException:") != -1)
                {
                    Alert.show(faultEvent.fault.faultString, this.resourceManager.common_error);
                }
                else if (faultEvent.fault.faultDetail.indexOf("InvalidProjectIdException:") != -1)
                {
                    Alert.show(this.resourceManager.errorMessage_invalidProjectIdExceptionMessage, this.resourceManager.common_error);
                }
                else if (faultEvent.fault.faultDetail.indexOf("InvalidNameException:") != -1)
                {
                    Alert.show(this.resourceManager.errorMessage_invalidNameExceptionMessage, this.resourceManager.common_error);
                }
                else if (faultEvent.fault.faultDetail.indexOf("InvalidMachineImageException:") != -1)
                {
                    Alert.show(this.resourceManager.errorMessage_invalidVMImageExceptionMessage, this.resourceManager.common_error);
                }
                else if (faultEvent.fault.faultDetail.indexOf("MachineImageInUseException:") != -1)
                {
                    Alert.show(this.resourceManager.errorMessage_VMImageInUseExceptionMessage, this.resourceManager.common_error);
                }
                else if (faultEvent.fault.faultDetail.indexOf("ResourceQuotaExceededException:") != -1)
                {
                    Alert.show(this.resourceManager.errorMessage_resourceQuotaExceededExceptionMessage, this.resourceManager.common_error);
                }
                else if (faultEvent.fault.faultDetail.indexOf("DuplicateNameException:") != -1)
                {
                    Alert.show(faultEvent.fault.faultString, this.resourceManager.common_error);
                }
                else if (faultEvent.fault.faultDetail.indexOf("InvalidMachineConfigurationException:") != -1)
                {
                    Alert.show(this.resourceManager.errorMessage_invalidVirtualMachineSizeExceptionMessage, this.resourceManager.common_error);
                }
                else if (faultEvent.fault.faultDetail.indexOf("InvalidMachineIdException:") != -1)
                {
                    Alert.show(this.resourceManager.errorMessage_invalidVirtualMachineIdExceptionMessage, this.resourceManager.common_error);
                }
                else if (faultEvent.fault.faultDetail.indexOf("InvalidPowerStateException:") != -1)
                {
                    Alert.show(this.resourceManager.errorMessage_invalidPowerStateExceptionMessage, this.resourceManager.common_error);
                }
                else if (faultEvent.fault.faultDetail.indexOf("InvalidHostIdException:") != -1)
                {
                    Alert.show(this.resourceManager.errorMessage_invalidHostIdExceptionMessage, this.resourceManager.common_error);
                }
                else if (faultEvent.fault.faultDetail.indexOf("InvalidPerfMetricIdException:") != -1)
                {
                    Alert.show(this.resourceManager.errorMessage_invalidPerfMetricIdExceptionMessage, this.resourceManager.common_error);
                }
                else if (faultEvent.fault.faultDetail.indexOf("InsufficientResourcesException:") != -1)
                {
                    Alert.show(this.resourceManager.errorMessage_invalidPerfMetricIdExceptionMessage, this.resourceManager.common_error);
                }
                else if (faultEvent.fault.faultDetail.indexOf("BadVMPowerStateException:") != -1)
                {
                    //TODO: Vérifier si on reçoit ce type d'exception
                }
                else if (faultEvent.fault.faultDetail.indexOf("CloudProviderException:") != -1)
                {
                    if (faultEvent.fault.faultDetail.indexOf("migrateVirtualMachine") >= 0)
                    {
                        // Error during the migration
                        vmMigrationFailedWatcher++;
                    }
                    else
                    {
                        Alert.show(faultEvent.fault.faultString, this.resourceManager.common_error);
                    }
                }
                else
                {
                    // Unknown error
                    if (faultEvent.fault.faultDetail.indexOf("NetConnection.Call.Failed") != -1)
                    {
                        Alert.show("Communication failure with server", "Info");
                    }
                    else
                        // General error
                        Alert.show("Error: " + faultEvent.fault.faultString, this.resourceManager.common_error);
                    
                }
            }
        }
        
        // Cairngorm
        
        private var _authenticateUserDelegate : AuthenticateUserDelegate = new AuthenticateUserDelegate();
        
        private var _userLogoutDelegate : UserLogoutDelegate = new UserLogoutDelegate();
        
        private var _createVirtualMachineDelegate : CreateVirtualMachineDelegate = new CreateVirtualMachineDelegate();
        
        private var _createVMImageDelegate : CreateVMImageDelegate = new CreateVMImageDelegate();
        
        private var _createProjectDelegate : CreateProjectDelegate = new CreateProjectDelegate();
        
        private var _createUserDelegate : CreateUserDelegate = new CreateUserDelegate();
        
        private var _destroyVirtualMachineDelegate : DestroyVirtualMachineDelegate = new DestroyVirtualMachineDelegate();
        
        private var _destroyVMImageDelegate : DestroyVMImageDelegate = new DestroyVMImageDelegate();
        
        private var _destroyProjectDelegate : DestroyProjectDelegate = new DestroyProjectDelegate();
        
        private var _migrateVirtualMachineDelegate : MigrateVirtualMachineDelegate = new MigrateVirtualMachineDelegate();
        
        private var _getAllVirtualMachinesDelegate : GetAllVirtualMachinesDelegate = new GetAllVirtualMachinesDelegate();
        
        private var _getVirtualMachinesByProjectDelegate : GetVirtualMachinesByProjectDelegate = new GetVirtualMachinesByProjectDelegate();
        
        private var _getAllProjectsDelegate : GetAllProjectsDelegate = new GetAllProjectsDelegate();
        
        private var _getProjectsByUserDelegate : GetProjectsByUserDelegate = new GetProjectsByUserDelegate();
        
        private var _getAllVMImagesDelegate : GetAllVMImagesDelegate = new GetAllVMImagesDelegate();
        
        private var _getAllUsersDelegate : GetAllUsersDelegate = new GetAllUsersDelegate();
        
        private var _getDataCenterLocationsDelegate : GetCloudProviderLocationsDelegate = new GetCloudProviderLocationsDelegate();
        
        private var _getUserByUsernameDelegate : GetUserByUsernameDelegate = new GetUserByUsernameDelegate();
        
        private var _getUserLdapInfoByUsernameDelegate : GetUserLdapInfoByUsernameDelegate = new GetUserLdapInfoByUsernameDelegate();
        
        private var _getProjectResourceConsumptionDelegate : GetProjectResourceConsumptionDelegate = new GetProjectResourceConsumptionDelegate();
        
        private var _getVirtualMachineByVmIdDelegate : GetVirtualMachineByVmIdDelegate = new GetVirtualMachineByVmIdDelegate();
        
        private var _getVMImagesByProjectDelegate : GetVMImagesByProjectDelegate = new GetVMImagesByProjectDelegate();
        
        private var _getVMSizeNameDelegate : GetVMSizeNameDelegate = new GetVMSizeNameDelegate();
        
        private var _getVMSizesDelegate : GetVMSizesDelegate = new GetVMSizesDelegate();
        
        private var _rebootVirtualMachineDelegate : RebootVirtualMachineDelegate = new RebootVirtualMachineDelegate();
        
        private var _updateVirtualMachineExpirationDateDelegate : UpdateVirtualMachineExpirationDateDelegate = new UpdateVirtualMachineExpirationDateDelegate();
        
        private var _startVirtualMachineDelegate : StartVirtualMachineDelegate = new StartVirtualMachineDelegate();
        
        private var _stopVirtualMachineDelegate : StopVirtualMachineDelegate = new StopVirtualMachineDelegate();
        
        private var _pauseVirtualMachineDelegate : PauseVirtualMachineDelegate = new PauseVirtualMachineDelegate();
        
        private var _unpauseVirtualMachineDelegate : UnpauseVirtualMachineDelegate = new UnpauseVirtualMachineDelegate();
        
        private var _updateUserSSHKeyDelegate : UpdateUserSSHKeyDelegate = new UpdateUserSSHKeyDelegate();
        
        private var _getProjectResourceQuotaDelegate : GetProjectResourceQuotaDelegate = new GetProjectResourceQuotaDelegate();
        
        private var _updateProjectResourceQuotaDelegate : UpdateProjectResourceQuotaDelegate = new UpdateProjectResourceQuotaDelegate();
        
        private var _updateUserPasswordsDelegate : UpdateUserPasswordsDelegate = new UpdateUserPasswordsDelegate();
        
        private var _updateVMImageDelegate : UpdateVMImageDelegate = new UpdateVMImageDelegate();
        
        private var _addUserToProjectDelegate : AddUserToProjectDelegate = new AddUserToProjectDelegate();
        
        private var _getVirtualMachineConsoleDelegate : GetVirtualMachineConsoleDelegate = new GetVirtualMachineConsoleDelegate();
        
        private var _getVirtualMachinesByHostDelegate : GetVirtualMachinesByHostDelegate = new GetVirtualMachinesByHostDelegate();
        
        private var _getDataCenterResourceTreeDelegate : GetDataCenterResourceTreeDelegate = new GetDataCenterResourceTreeDelegate();
        
        private var _getVirtualMachineExtendedInfoDelegate : GetVirtualMachineExtendedInfoDelegate = new GetVirtualMachineExtendedInfoDelegate();
        
        private var _getVirtualMachinesCPULoadDelegate : GetVirtualMachinesCPULoadDelegate = new GetVirtualMachinesCPULoadDelegate();
        
        private var _getVirtualMachinesMemoryUsedDelegate : GetVirtualMachinesMemoryUsedDelegate = new GetVirtualMachinesMemoryUsedDelegate();
        
        private var _getHostPerfMetricInfosDelegate : GetHostPerfMetricInfosDelegate = new GetHostPerfMetricInfosDelegate();
        
        private var _getVirtualMachinePerfMetricInfosDelegate : GetVirtualMachinePerfMetricInfosDelegate = new GetVirtualMachinePerfMetricInfosDelegate();
        
        private var _getHostPerfMetricsDelegate : GetHostPerfMetricsDelegate = new GetHostPerfMetricsDelegate();
        
        private var _getVirtualMachinePerfMetricsDelegate : GetVirtualMachinePerfMetricsDelegate = new GetVirtualMachinePerfMetricsDelegate();
        
        private var _deleteUserDelegate : DeleteUserDelegate = new DeleteUserDelegate();
        
        private var _deleteUserFromProjectDelegate : DeleteUserFromProjectDelegate = new DeleteUserFromProjectDelegate();
        
        private var _findEventsDelegate : FindEventsDelegate = new FindEventsDelegate();
        
        private var _getDbParametersDelegate : GetDbParametersDelegate = new GetDbParametersDelegate();
        
        private var _createVolumeDelegate : CreateVolumeDelegate = new CreateVolumeDelegate();
        
        private var _destroyVolumeDelegate : DestroyVolumeDelegate = new DestroyVolumeDelegate();
        
        private var _getVolumesByProjectDelegate : GetVolumesByProjectDelegate = new GetVolumesByProjectDelegate();
        
        private var _getAllVolumesDelegate : GetAllVolumesDelegate = new GetAllVolumesDelegate();
        
        private var _attachVolumeToVirtualMachineDelegate : AttachVolumeToVirtualMachineDelegate = new AttachVolumeToVirtualMachineDelegate();
        
        private var _detachVolumeFromVirtualMachineDelegate : DetachVolumeFromVirtualMachineDelegate = new DetachVolumeFromVirtualMachineDelegate();
        
        private var _getCloudProviderAccountsByProjectDelegate : GetCloudProviderAccountsByProjectDelegate = new GetCloudProviderAccountsByProjectDelegate();
        
        private var _getCloudProvidersDelegate : GetCloudProvidersDelegate = new GetCloudProvidersDelegate();
        
        
        
        public function get getDbParametersDelegate() : GetDbParametersDelegate
        {
            return _getDbParametersDelegate;
        }
        
        
        
        public function set getDbParametersDelegate(value : GetDbParametersDelegate) : void
        {
            _getDbParametersDelegate = value;
        }
        
        
        
        public function get authenticateUserDelegate() : AuthenticateUserDelegate
        {
            return _authenticateUserDelegate;
        }
        
        
        
        public function get userLogoutDelegate() : UserLogoutDelegate
        {
            return _userLogoutDelegate;
        }
        
        
        
        public function get createVirtualMachineDelegate() : CreateVirtualMachineDelegate
        {
            return _createVirtualMachineDelegate;
        }
        
        
        
        public function get createVMImageDelegate() : CreateVMImageDelegate
        {
            return _createVMImageDelegate;
        }
        
        
        
        public function get destroyVirtualMachineDelegate() : DestroyVirtualMachineDelegate
        {
            return _destroyVirtualMachineDelegate;
        }
        
        
        
        public function get destroyProjectDelegate() : DestroyProjectDelegate
        {
            return _destroyProjectDelegate;
        }
        
        
        
        public function get destroyVMImageDelegate() : DestroyVMImageDelegate
        {
            return _destroyVMImageDelegate;
        }
        
        
        
        public function get migrateVirtualMachineDelegate() : MigrateVirtualMachineDelegate
        {
            return _migrateVirtualMachineDelegate;
        }
        
        
        
        public function get getAllVirtualMachinesDelegate() : GetAllVirtualMachinesDelegate
        {
            return _getAllVirtualMachinesDelegate;
        }
        
        
        
        public function get getVirtualMachinesByProjectDelegate() : GetVirtualMachinesByProjectDelegate
        {
            return _getVirtualMachinesByProjectDelegate;
        }
        
        
        
        public function get getAllVMImagesDelegate() : GetAllVMImagesDelegate
        {
            return _getAllVMImagesDelegate;
        }
        
        
        
        public function get getAllProjectsDelegate() : GetAllProjectsDelegate
        {
            return _getAllProjectsDelegate;
        }
        
        
        
        public function get getAllUsersDelegate() : GetAllUsersDelegate
        {
            return _getAllUsersDelegate;
        }
        
        
        
        public function get getDataCenterLocationsDelegate() : GetCloudProviderLocationsDelegate
        {
            return _getDataCenterLocationsDelegate;
        }
        
        
        
        public function get getUserByUsernameDelegate() : GetUserByUsernameDelegate
        {
            return _getUserByUsernameDelegate;
        }
        
        
        
        public function get getUserLdapInfoByUsernameDelegate() : GetUserLdapInfoByUsernameDelegate
        {
            return _getUserLdapInfoByUsernameDelegate;
        }
        
        
        
        public function get getProjectResourceConsumptionDelegate() : GetProjectResourceConsumptionDelegate
        {
            return _getProjectResourceConsumptionDelegate;
        }
        
        
        
        public function get getVirtualMachineByVmIdDelegate() : GetVirtualMachineByVmIdDelegate
        {
            return _getVirtualMachineByVmIdDelegate;
        }
        
        
        
        public function get getVMImagesByProjectDelegate() : GetVMImagesByProjectDelegate
        {
            return _getVMImagesByProjectDelegate;
        }
        
        
        
        public function get getProjectsByUserDelegate() : GetProjectsByUserDelegate
        {
            return _getProjectsByUserDelegate;
        }
        
        
        
        public function get getVMSizeNameDelegate() : GetVMSizeNameDelegate
        {
            return _getVMSizeNameDelegate;
        }
        
        
        
        public function get getVMSizesDelegate() : GetVMSizesDelegate
        {
            return _getVMSizesDelegate;
        }
        
        
        
        public function get rebootVirtualMachineDelegate() : RebootVirtualMachineDelegate
        {
            return _rebootVirtualMachineDelegate;
        }
        
        
        
        public function get startVirtualMachineDelegate() : StartVirtualMachineDelegate
        {
            return _startVirtualMachineDelegate;
        }
        
        
        
        public function get stopVirtualMachineDelegate() : StopVirtualMachineDelegate
        {
            return _stopVirtualMachineDelegate;
        }
        
        
        
        public function get pauseVirtualMachineDelegate() : PauseVirtualMachineDelegate
        {
            return _pauseVirtualMachineDelegate;
        }
        
        
        
        public function get unpauseVirtualMachineDelegate() : UnpauseVirtualMachineDelegate
        {
            return _unpauseVirtualMachineDelegate;
        }
        
        
        
        public function get updateUserSSHKeyDelegate() : UpdateUserSSHKeyDelegate
        {
            return _updateUserSSHKeyDelegate;
        }
        
        
        
        public function get getProjectResourceQuotaDelegate() : GetProjectResourceQuotaDelegate
        {
            return _getProjectResourceQuotaDelegate;
        }
        
        
        
        public function get updateProjectResourceQuotaDelegate() : UpdateProjectResourceQuotaDelegate
        {
            return _updateProjectResourceQuotaDelegate;
        }
        
        
        
        public function get updateUserPasswordsDelegate() : UpdateUserPasswordsDelegate
        {
            return _updateUserPasswordsDelegate;
        }
        
        
        
        public function get updateVMImageDelegate() : UpdateVMImageDelegate
        {
            return _updateVMImageDelegate;
        }
        
        
        
        public function get updateProjectDelegate() : AddUserToProjectDelegate
        {
            return _addUserToProjectDelegate;
        }
        
        
        
        public function get getVirtualMachineConsoleDelegate() : GetVirtualMachineConsoleDelegate
        {
            return _getVirtualMachineConsoleDelegate;
        }
        
        
        
        public function get getVirtualMachinesByHostDelegate() : GetVirtualMachinesByHostDelegate
        {
            return _getVirtualMachinesByHostDelegate;
        }
        
        
        
        public function get getDataCenterResourceTreeDelegate() : GetDataCenterResourceTreeDelegate
        {
            return _getDataCenterResourceTreeDelegate;
        }
        
        
        
        public function get getVirtualMachineExtendedInfoDelegate() : GetVirtualMachineExtendedInfoDelegate
        {
            return _getVirtualMachineExtendedInfoDelegate;
        }
        
        
        
        public function get getVirtualMachinesCPULoadDelegate() : GetVirtualMachinesCPULoadDelegate
        {
            return _getVirtualMachinesCPULoadDelegate;
        }
        
        
        
        public function get getVirtualMachinesMemoryUsedDelegate() : GetVirtualMachinesMemoryUsedDelegate
        {
            return _getVirtualMachinesMemoryUsedDelegate;
        }
        
        
        
        public function get getHostPerfMetricInfosDelegate() : GetHostPerfMetricInfosDelegate
        {
            return _getHostPerfMetricInfosDelegate;
        }
        
        
        
        public function get getVirtualMachinePerfMetricInfosDelegate() : GetVirtualMachinePerfMetricInfosDelegate
        {
            return _getVirtualMachinePerfMetricInfosDelegate;
        }
        
        
        
        public function get getHostPerfMetricsDelegate() : GetHostPerfMetricsDelegate
        {
            return _getHostPerfMetricsDelegate;
        }
        
        
        
        public function get getVirtualMachinePerfMetricsDelegate() : GetVirtualMachinePerfMetricsDelegate
        {
            return _getVirtualMachinePerfMetricsDelegate;
        }
        
        
        
        public function get deleteUserDelegate() : DeleteUserDelegate
        {
            return _deleteUserDelegate;
        }
        
        
        
        public function get findEventsDelegate() : FindEventsDelegate
        {
            return _findEventsDelegate;
        }
        
        
        
        public function get createProjectDelegate() : CreateProjectDelegate
        {
            return _createProjectDelegate;
        }
        
        
        
        public function get createUserDelegate() : CreateUserDelegate
        {
            return _createUserDelegate;
        }
        
        
        
        public function get addUserToProjectDelegate() : AddUserToProjectDelegate
        {
            return _addUserToProjectDelegate;
        }
        
        
        
        public function set addUserToProjectDelegate(value : AddUserToProjectDelegate) : void
        {
            _addUserToProjectDelegate = value;
        }
        
        
        
        public function get deleteUserFromProjectDelegate() : DeleteUserFromProjectDelegate
        {
            return _deleteUserFromProjectDelegate;
        }
        
        
        
        public function set deleteUserFromProjectDelegate(value : DeleteUserFromProjectDelegate) : void
        {
            _deleteUserFromProjectDelegate = value;
        }
        
        
        
        public function get updateVirtualMachineExpirationDateDelegate() : UpdateVirtualMachineExpirationDateDelegate
        {
            return _updateVirtualMachineExpirationDateDelegate;
        }
        
        
        
        public function set updateVirtualMachineExpirationDateDelegate(value : UpdateVirtualMachineExpirationDateDelegate) : void
        {
            _updateVirtualMachineExpirationDateDelegate = value;
        }
        
        
        
        public function get detachVolumeFromVirtualMachineDelegate() : DetachVolumeFromVirtualMachineDelegate
        {
            return _detachVolumeFromVirtualMachineDelegate;
        }
        
        
        
        public function get attachVolumeToVirtualMachineDelegate() : AttachVolumeToVirtualMachineDelegate
        {
            return _attachVolumeToVirtualMachineDelegate;
        }
        
        
        
        public function get getVolumesByProjectDelegate() : GetVolumesByProjectDelegate
        {
            return _getVolumesByProjectDelegate;
        }
        
        
        
        public function get destroyVolumeDelegate() : DestroyVolumeDelegate
        {
            return _destroyVolumeDelegate;
        }
        
        
        
        public function get createVolumeDelegate() : CreateVolumeDelegate
        {
            return _createVolumeDelegate;
        }
        
        
        
        public function get getAllVolumesDelegate() : GetAllVolumesDelegate
        {
            return _getAllVolumesDelegate;
        }
        
        public function get getCloudProviderAccountsByProjectDelegate() : GetCloudProviderAccountsByProjectDelegate
        {
            return _getCloudProviderAccountsByProjectDelegate;
        }
        
        public function get getCloudProvidersDelegate() : GetCloudProvidersDelegate
        {
            return _getCloudProvidersDelegate;
        }
    }
}


/**
 * Inner class used for the Singleton Design Pattern.
 * @author ARLERAY
 */
class SingletonEnforcer
{

}

