/**
 *
 * SIROCCO
 * Copyright (C) 2011 France Telecom
 * Contact: sirocco@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  $Id: OOCResourceManager.as 754 2011-12-15 18:20:47Z dangtran $
 *
 */

package org.ow2.sirocco.cloudmanager.utils.internationalization
{
    import flash.events.Event;
    import flash.utils.ByteArray;
    
    
    public class SiroccoResourceManager
    {
        // Single instance of our Resource Manager
        private static var instance : SiroccoResourceManager;
        
        /**
         * The locale representing the current language,
         * formatted as the language code in lower case followed by the country code in upper case.
         * e.g. "en_US" for "english as used in the United States",
         * or "fr_FR" for "french as used in France".
         */
        private var _locale : String;
        
        
        
        [Bindable]
        public function get locale() : String
        {
            return this._locale;
        }
        
        
        
        public function set locale(newLocale : String) : void
        {
            this._locale = newLocale;
            initValues();
        }
        
        private var xmlTranslations : XML;
        
        //		[Embed(source="config/translations.xml", mimeType="text/xml")]
        [Embed(source="/org/ow2/sirocco/cloudmanager/client/config/translations.xml", mimeType="application/octet-stream")]
        protected const EmbeddedXmlTranslations : Class;
        
        
        
        /**
         * Returns a unique instance of SiroccoResourceManager (Singleton Design Pattern).
         * @return The unique instance of SiroccoResourceManager.
         */
        public static function getInstance() : SiroccoResourceManager
        {
            if (instance == null)
            {
                instance = new SiroccoResourceManager(new SingletonEnforcer());
            }
            
            return instance;
        }
        
        
        
        public function SiroccoResourceManager(enforcer : SingletonEnforcer)
        {
            if (enforcer == null)
            {
                throw new Error("SiroccoResourceManager constructor forbidden! Instead, use SiroccoResourceManager.getInstance().");
            }
            else
            {
                var byteArray : ByteArray = new EmbeddedXmlTranslations() as ByteArray;
                this.xmlTranslations = new XML(byteArray.readUTFBytes(byteArray.length));
                onResourcesLoaded(null);
            }
        }
        
        
        
        private function onResourcesLoaded(event : Event) : void
        {
            this.locale = "en_US";
        }
        
        [Bindable]
        public var common_loading : String;
        
        [Bindable]
        public var common_ok : String;
        
        [Bindable]
        public var common_select : String;
        
        [Bindable]
        public var common_cancel : String;
        
        [Bindable]
        public var common_next : String;
        
        [Bindable]
        public var common_previous : String;
        
        [Bindable]
        public var common_finish : String;
        
        [Bindable]
        public var common_summary : String;
        
        [Bindable]
        public var common_error : String;
        
        [Bindable]
        public var common_info : String;
        
        [Bindable]
        public var common_confirm : String;
        
        [Bindable]
        public var common_refresh : String;
        
        [Bindable]
        public var common_refreshing : String;
        
        [Bindable]
        public var common_filter : String;
        
        [Bindable]
        public var common_collapseAll : String;
        
        [Bindable]
        public var common_expandAll : String;
        
        [Bindable]
        public var common_vmImageSuccessfullyCreated : String;
        
        [Bindable]
        public var main_title : String;
        
        [Bindable]
        public var main_titleSeparator : String;
        
        [Bindable]
        public var main_help : String;
        
        [Bindable]
        public var main_logout : String;
        
        [Bindable]
        public var main_username : String;
        
        [Bindable]
        public var main_tabVirtualMachines : String;
        
        [Bindable]
        public var main_tabVMImages : String;
        
        [Bindable]
        public var main_tabAccount : String;
        
        [Bindable]
        public var main_tabInventory : String;
        
        [Bindable]
        public var main_tabImageManagement : String;
        
        [Bindable]
        public var main_tabUserManagement : String;
        
        [Bindable]
        public var errorMessage_invalidUsernameExceptionMessage : String;
        
        [Bindable]
        public var errorMessage_invalidProjectIdExceptionMessage : String;
        
        [Bindable]
        public var errorMessage_invalidNameExceptionMessage : String;
        
        [Bindable]
        public var errorMessage_invalidVMImageExceptionMessage : String;
        
        [Bindable]
        public var errorMessage_VMImageInUseExceptionMessage : String;
        
        [Bindable]
        public var errorMessage_resourceQuotaExceededExceptionMessage : String;
        
        [Bindable]
        public var errorMessage_duplicateNameExceptionMessage : String;
        
        [Bindable]
        public var errorMessage_invalidVirtualMachineSizeExceptionMessage : String;
        
        [Bindable]
        public var errorMessage_invalidVirtualMachineIdExceptionMessage : String;
        
        [Bindable]
        public var errorMessage_cloudAdminExceptionMessage : String;
        
        [Bindable]
        public var errorMessage_invalidPowerStateExceptionMessage : String;
        
        [Bindable]
        public var errorMessage_invalidHostIdExceptionMessage : String;
        
        [Bindable]
        public var errorMessage_invalidPerfMetricIdExceptionMessage : String;
        
        [Bindable]
        public var errorMessage_vmMigrationFailedExceptionMessage : String;
        
        [Bindable]
        public var userLogin_title : String;
        
        [Bindable]
        public var userLogin_login : String;
        
        [Bindable]
        public var userLogin_password : String;
        
        [Bindable]
        public var userLogin_btnLogin : String;
        
        [Bindable]
        public var userLogin_btnCreateAccount : String;
        
        [Bindable]
        public var adminLogin_title : String;
        
        [Bindable]
        public var adminLogin_login : String;
        
        [Bindable]
        public var adminLogin_password : String;
        
        [Bindable]
        public var adminLogin_btnLogin : String;
        
        [Bindable]
        public var adminLogin_btnCreateAccount : String;
        
        [Bindable]
        public var virtualMachines_title : String;
        
        [Bindable]
        public var virtualMachines_btnRefresh : String;
        
        [Bindable]
        public var virtualMachines_btnRefreshing : String;
        
        [Bindable]
        public var virtualMachines_btnNewVM : String;
        
        [Bindable]
        public var virtualMachines_btnDestroyVM : String;
        
        [Bindable]
        public var virtualMachines_destroyVMConfirm : String;
        
        [Bindable]
        public var virtualMachines_btnStartVM : String;
        
        [Bindable]
        public var virtualMachines_btnStopVM : String;
        
        [Bindable]
        public var virtualMachines_btnRebootVM : String;
        
        [Bindable]
        public var virtualMachines_btnMakeImage : String;
        
        [Bindable]
        public var virtualMachines_btnVNC : String;
        
        [Bindable]
        public var virtualMachines_colVMid : String;
        
        [Bindable]
        public var virtualMachines_colVMName : String;
        
        [Bindable]
        public var virtualMachines_colStatus : String;
        
        [Bindable]
        public var virtualMachines_colIPAddress : String;
        
        [Bindable]
        public var virtualMachines_colDns : String;
        
        [Bindable]
        public var virtualMachines_colCreationDate : String;
        
        [Bindable]
        public var virtualMachines_colImage : String;
        
        [Bindable]
        public var virtualMachines_colMemory : String;
        
        [Bindable]
        public var virtualMachines_colDiskSize : String;
        
        [Bindable]
        public var virtualMachines_colNbCpu : String;
        
        [Bindable]
        public var virtualMachines_makeImagePopupTitle : String;
        
        [Bindable]
        public var virtualMachines_makeImagePopupInstructions : String;
        
        [Bindable]
        public var virtualMachines_makeImagePopupName : String;
        
        [Bindable]
        public var virtualMachines_makeImagePopupDescription : String;
        
        [Bindable]
        public var virtualMachines_makeImagePopupButton : String;
        
        [Bindable]
        public var virtualMachines_msgVncOk : String;
        
        [Bindable]
        public var virtualMachines_msgVncNull : String;
        
        [Bindable]
        public var userCreateVM_title : String;
        
        [Bindable]
        public var userCreateVM_VMLabel : String;
        
        [Bindable]
        public var userCreateVM_VMHostName : String;
        
        [Bindable]
        public var userCreateVM_image : String;
        
        [Bindable]
        public var userCreateVM_nbCPU : String;
        
        [Bindable]
        public var userCreateVM_memoryCapacity : String;
        
        [Bindable]
        public var userCreateVM_diskSize : String;
        
        [Bindable]
        public var userCreateVM_location : String;
        
        [Bindable]
        public var userCreateVM_btnChangeImage : String;
        
        [Bindable]
        public var userCreateVM_btnCreate : String;
        
        [Bindable]
        public var userCreateVM_btnCancel : String;
        
        [Bindable]
        public var userCreateVM_msgVMLabelMandatory : String;
        
        [Bindable]
        public var userCreateVM_msgVMLabelInvalid : String;
        
        [Bindable]
        public var userCreateVM_msgVMImageMandatory : String;
        
        [Bindable]
        public var vmImageBrowser_title : String;
        
        [Bindable]
        public var vmImageBrowser_category : String;
        
        [Bindable]
        public var vmImageBrowser_hypervisor : String;
        
        [Bindable]
        public var vmImageBrowser_myImages : String;
        
        [Bindable]
        public var vmImageBrowser_publicImages : String;
        
        [Bindable]
        public var vmImageBrowser_colCategory : String;
        
        [Bindable]
        public var vmImageBrowser_colName : String;
        
        [Bindable]
        public var vmImageBrowser_colDescription : String;
        
        [Bindable]
        public var vmImageBrowser_colHypervisor : String;
        
        [Bindable]
        public var vmImageBrowser_colArchitecture : String;
        
        [Bindable]
        public var vmImageBrowser_colDiskSizeMB : String;
        
        [Bindable]
        public var commonVMImages_titleUser : String;
        
        [Bindable]
        public var commonVMImages_titleAdmin : String;
        
        [Bindable]
        public var commonVMImages_btnDestroyVMImages : String;
        
        [Bindable]
        public var commonVMImages_destroyVMImagesConfirm : String;
        
        [Bindable]
        public var commonVMImages_userFilter : String;
        
        [Bindable]
        public var commonVMImages_categoryFilter : String;
        
        [Bindable]
        public var commonVMImages_hypervisorFilter : String;
        
        [Bindable]
        public var commonVMImages_colId : String;
        
        [Bindable]
        public var commonVMImages_colStatus : String;
        
        [Bindable]
        public var commonVMImages_colUser : String;
        
        [Bindable]
        public var commonVMImages_colCategory : String;
        
        [Bindable]
        public var commonVMImages_colName : String;
        
        [Bindable]
        public var commonVMImages_colDescription : String;
        
        [Bindable]
        public var commonVMImages_colHypervisor : String;
        
        [Bindable]
        public var commonVMImages_colArchitecture : String;
        
        [Bindable]
        public var commonVMImages_colDiskSizeMB : String;
        
        [Bindable]
        public var userView_title : String;
        
        [Bindable]
        public var userView_infos : String;
        
        [Bindable]
        public var userView_lastName : String;
        
        [Bindable]
        public var userView_firstName : String;
        
        [Bindable]
        public var userView_username : String;
        
        [Bindable]
        public var userView_email : String;
        
        [Bindable]
        public var userView_quotas : String;
        
        [Bindable]
        public var userView_free : String;
        
        [Bindable]
        public var userView_used : String;
        
        [Bindable]
        public var userView_cpuQuota : String;
        
        [Bindable]
        public var userView_ramQuota : String;
        
        [Bindable]
        public var userView_diskQuota : String;
        
        [Bindable]
        public var userView_nbVMQuota : String;
        
        [Bindable]
        public var userView_cpuConsumption : String;
        
        [Bindable]
        public var userView_ramConsumption : String;
        
        [Bindable]
        public var userView_diskConsumption : String;
        
        [Bindable]
        public var userView_nbVMConsumption : String;
        
        [Bindable]
        public var userView_SSHKey : String;
        
        [Bindable]
        public var userView_currentSSHKey : String;
        
        [Bindable]
        public var userView_newSSHKey : String;
        
        [Bindable]
        public var userView_updateSSHKey : String;
        
        [Bindable]
        public var userView_passwords : String;
        
        [Bindable]
        public var userView_updateWebPassword : String;
        
        [Bindable]
        public var userView_webPassword : String;
        
        [Bindable]
        public var userView_newWebPassword : String;
        
        [Bindable]
        public var userView_newWebPassword2 : String;
        
        [Bindable]
        public var userView_updateAPIPassword : String;
        
        [Bindable]
        public var userView_APIPassword : String;
        
        [Bindable]
        public var userView_newAPIPassword : String;
        
        [Bindable]
        public var userView_newAPIPassword2 : String;
        
        [Bindable]
        public var userView_newPasswordError : String;
        
        [Bindable]
        public var userView_oldPasswordError : String;
        
        [Bindable]
        public var inventory_title : String;
        
        [Bindable]
        public var inventory_btnListOnly : String;
        
        [Bindable]
        public var inventory_btnSplit : String;
        
        [Bindable]
        public var inventory_btnDetailOnly : String;
        
        [Bindable]
        public var inventory_colUser : String;
        
        [Bindable]
        public var inventory_colVMName : String;
        
        [Bindable]
        public var inventory_colStatus : String;
        
        [Bindable]
        public var inventory_colIPAddress : String;
        
        [Bindable]
        public var inventory_colDns : String;
        
        [Bindable]
        public var inventory_colImage : String;
        
        [Bindable]
        public var inventory_colMemory : String;
        
        [Bindable]
        public var inventory_colDiskSize : String;
        
        [Bindable]
        public var inventory_colNbCpu : String;
        
        [Bindable]
        public var inventory_colCpuLoad : String;
        
        [Bindable]
        public var inventory_colMemoryUsed : String;
        
        [Bindable]
        public var inventory_btnDestroyVM : String;
        
        [Bindable]
        public var inventory_destroyVMConfirm : String;
        
        [Bindable]
        public var inventory_btnStartVM : String;
        
        [Bindable]
        public var inventory_btnStopVM : String;
        
        [Bindable]
        public var inventory_btnRebootVM : String;
        
        [Bindable]
        public var inventory_btnPauseVM : String;
        
        [Bindable]
        public var inventory_btnUnpauseVM : String;
        
        [Bindable]
        public var inventory_btnConsoleVM : String;
        
        [Bindable]
        public var inventory_user : String;
        
        [Bindable]
        public var inventory_dataCenter : String;
        
        [Bindable]
        public var inventory_domain : String;
        
        [Bindable]
        public var inventory_serverPool : String;
        
        [Bindable]
        public var inventory_host : String;
        
        [Bindable]
        public var inventory_virtualMachine : String;
        
        [Bindable]
        public var inventoryDomain_domain : String;
        
        [Bindable]
        public var inventoryServerPool_serverPool : String;
        
        [Bindable]
        public var inventoryHost_host : String;
        
        [Bindable]
        public var inventoryVirtualMachine_virtualMachine : String;
        
        [Bindable]
        public var inventoryVirtualMachine_tabCpuScheduling : String;
        
        [Bindable]
        public var inventoryVirtualMachine_tabNetworkInterfaces : String;
        
        [Bindable]
        public var inventoryVirtualMachine_tabPerformances : String;
        
        [Bindable]
        public var inventoryVirtualMachine_tabConsole : String;
        
        [Bindable]
        public var inventoryVirtualMachine_schedulingWeight : String;
        
        [Bindable]
        public var inventoryVirtualMachine_schedulingCap : String;
        
        [Bindable]
        public var inventoryVirtualMachine_colDeviceName : String;
        
        [Bindable]
        public var inventoryVirtualMachine_colMacAddress : String;
        
        [Bindable]
        public var inventoryVirtualMachine_vncConsole : String;
        
        [Bindable]
        public var inventoryVirtualMachine_msgConsoleNotAvailable : String;
        
        [Bindable]
        public var performances_cbPerfMetricsPrompt : String;
        
        [Bindable]
        public var performances_cbStacked : String;
        
        [Bindable]
        public var performances_cbPast30Seconds : String;
        
        [Bindable]
        public var performances_cbPastMinute : String;
        
        [Bindable]
        public var performances_cbPast5Minutes : String;
        
        [Bindable]
        public var performances_cbPast30Minutes : String;
        
        [Bindable]
        public var performances_cbPastHour : String;
        
        [Bindable]
        public var performances_cbPastDay : String;
        
        [Bindable]
        public var performances_cbPastMonth : String;
        
        [Bindable]
        public var performances_cbPastYear : String;
        
        [Bindable]
        public var imageManagement_title : String;
        
        [Bindable]
        public var userManagement_title : String;
        
        [Bindable]
        public var userManagement_btnDeleteUser : String;
        
        [Bindable]
        public var userManagement_btnUpdateQuota : String;
        
        [Bindable]
        public var userManagement_btnCreateUser : String;
        
        [Bindable]
        public var userManagement_deleteUserConfirm : String;
        
        [Bindable]
        public var userManagement_colUsername : String;
        
        [Bindable]
        public var userManagement_colFirstName : String;
        
        [Bindable]
        public var userManagement_colLastName : String;
        
        [Bindable]
        public var userManagement_colEmail : String;
        
        [Bindable]
        public var userManagement_colPortalPassword : String;
        
        [Bindable]
        public var userManagement_colApiPassword : String;
        
        [Bindable]
        public var userManagement_updateQuota : String;
        
        [Bindable]
        public var userManagement_cpuQuota : String;
        
        [Bindable]
        public var userManagement_storageQuota : String;
        
        [Bindable]
        public var userManagement_memoryQuota : String;
        
        [Bindable]
        public var userManagement_vmQuota : String;
        
        
        
        private function initValues() : void
        {
            
            this.common_loading = getTranslation("common", "loading");
            this.common_ok = getTranslation("common", "ok");
            this.common_select = getTranslation("common", "select");
            this.common_cancel = getTranslation("common", "cancel");
            this.common_next = getTranslation("common", "next");
            this.common_previous = getTranslation("common", "previous");
            this.common_finish = getTranslation("common", "finish");
            this.common_summary = getTranslation("common", "summary");
            this.common_error = getTranslation("common", "error");
            this.common_info = getTranslation("common", "info");
            this.common_confirm = getTranslation("common", "confirm");
            this.common_refresh = getTranslation("common", "refresh");
            this.common_refreshing = getTranslation("common", "refreshing");
            this.common_filter = getTranslation("common", "filter");
            this.common_collapseAll = getTranslation("common", "collapseAll");
            this.common_expandAll = getTranslation("common", "expandAll");
            this.common_vmImageSuccessfullyCreated = getTranslation("common", "vmImageSuccessfullyCreated");
            this.main_title = getTranslation("main", "title");
            this.main_titleSeparator = getTranslation("main", "titleSeparator");
            this.main_help = getTranslation("main", "help");
            this.main_logout = getTranslation("main", "logout");
            this.main_username = getTranslation("main", "username");
            this.main_tabVirtualMachines = getTranslation("main", "tabVirtualMachines");
            this.main_tabVMImages = getTranslation("main", "tabVMImages");
            this.main_tabAccount = getTranslation("main", "tabAccount");
            this.main_tabInventory = getTranslation("main", "tabInventory");
            this.main_tabImageManagement = getTranslation("main", "tabImageManagement");
            this.main_tabUserManagement = getTranslation("main", "tabUserManagement");
            this.errorMessage_invalidUsernameExceptionMessage = getTranslation("errorMessage", "invalidUsernameExceptionMessage");
            this.errorMessage_invalidProjectIdExceptionMessage = getTranslation("errorMessage", "invalidProjectIdExceptionMessage");
            this.errorMessage_invalidNameExceptionMessage = getTranslation("errorMessage", "invalidNameExceptionMessage");
            this.errorMessage_invalidVMImageExceptionMessage = getTranslation("errorMessage", "invalidVMImageExceptionMessage");
            this.errorMessage_VMImageInUseExceptionMessage = getTranslation("errorMessage", "VMImageInUseExceptionMessage");
            this.errorMessage_resourceQuotaExceededExceptionMessage = getTranslation("errorMessage", "resourceQuotaExceededExceptionMessage");
            this.errorMessage_duplicateNameExceptionMessage = getTranslation("errorMessage", "duplicateNameExceptionMessage");
            this.errorMessage_invalidVirtualMachineSizeExceptionMessage = getTranslation("errorMessage", "invalidVirtualMachineSizeExceptionMessage");
            this.errorMessage_invalidVirtualMachineIdExceptionMessage = getTranslation("errorMessage", "invalidVirtualMachineIdExceptionMessage");
            this.errorMessage_cloudAdminExceptionMessage = getTranslation("errorMessage", "cloudAdminExceptionMessage");
            this.errorMessage_invalidPowerStateExceptionMessage = getTranslation("errorMessage", "invalidPowerStateExceptionMessage");
            this.errorMessage_invalidHostIdExceptionMessage = getTranslation("errorMessage", "invalidHostIdExceptionMessage");
            this.errorMessage_invalidPerfMetricIdExceptionMessage = getTranslation("errorMessage", "invalidPerfMetricIdExceptionMessage");
            this.errorMessage_vmMigrationFailedExceptionMessage = getTranslation("errorMessage", "vmMigrationFailedExceptionMessage");
            this.userLogin_title = getTranslation("userLogin", "title");
            this.userLogin_login = getTranslation("userLogin", "login");
            this.userLogin_password = getTranslation("userLogin", "password");
            this.userLogin_btnLogin = getTranslation("userLogin", "btnLogin");
            this.userLogin_btnCreateAccount = getTranslation("userLogin", "btnCreateAccount");
            this.adminLogin_title = getTranslation("adminLogin", "title");
            this.adminLogin_login = getTranslation("adminLogin", "login");
            this.adminLogin_password = getTranslation("adminLogin", "password");
            this.adminLogin_btnLogin = getTranslation("adminLogin", "btnLogin");
            this.adminLogin_btnCreateAccount = getTranslation("adminLogin", "btnCreateAccount");
            this.virtualMachines_title = getTranslation("virtualMachines", "title");
            this.virtualMachines_btnRefresh = getTranslation("virtualMachines", "btnRefresh");
            this.virtualMachines_btnRefreshing = getTranslation("virtualMachines", "btnRefreshing");
            this.virtualMachines_btnNewVM = getTranslation("virtualMachines", "btnNewVM");
            this.virtualMachines_btnDestroyVM = getTranslation("virtualMachines", "btnDestroyVM");
            this.virtualMachines_destroyVMConfirm = getTranslation("virtualMachines", "destroyVMConfirm");
            this.virtualMachines_btnStartVM = getTranslation("virtualMachines", "btnStartVM");
            this.virtualMachines_btnStopVM = getTranslation("virtualMachines", "btnStopVM");
            this.virtualMachines_btnRebootVM = getTranslation("virtualMachines", "btnRebootVM");
            this.virtualMachines_btnMakeImage = getTranslation("virtualMachines", "btnMakeImage");
            this.virtualMachines_btnVNC = getTranslation("virtualMachines", "btnVNC");
            this.virtualMachines_colVMid = getTranslation("virtualMachines", "colVMid");
            this.virtualMachines_colVMName = getTranslation("virtualMachines", "colVMName");
            this.virtualMachines_colStatus = getTranslation("virtualMachines", "colStatus");
            this.virtualMachines_colIPAddress = getTranslation("virtualMachines", "colIPAddress");
            this.virtualMachines_colDns = getTranslation("virtualMachines", "colDns");
            this.virtualMachines_colCreationDate = getTranslation("virtualMachines", "colCreationDate");
            this.virtualMachines_colImage = getTranslation("virtualMachines", "colImage");
            this.virtualMachines_colMemory = getTranslation("virtualMachines", "colMemory");
            this.virtualMachines_colDiskSize = getTranslation("virtualMachines", "colDiskSize");
            this.virtualMachines_colNbCpu = getTranslation("virtualMachines", "colNbCpu");
            this.virtualMachines_makeImagePopupTitle = getTranslation("virtualMachines", "makeImagePopupTitle");
            this.virtualMachines_makeImagePopupInstructions = getTranslation("virtualMachines", "makeImagePopupInstructions");
            this.virtualMachines_makeImagePopupName = getTranslation("virtualMachines", "makeImagePopupName");
            this.virtualMachines_makeImagePopupDescription = getTranslation("virtualMachines", "makeImagePopupDescription");
            this.virtualMachines_makeImagePopupButton = getTranslation("virtualMachines", "makeImagePopupButton");
            this.virtualMachines_msgVncOk = getTranslation("virtualMachines", "msgVncOk");
            this.virtualMachines_msgVncNull = getTranslation("virtualMachines", "msgVncNull");
            this.userCreateVM_title = getTranslation("userCreateVM", "title");
            this.userCreateVM_VMLabel = getTranslation("userCreateVM", "VMLabel");
            this.userCreateVM_VMHostName = getTranslation("userCreateVM", "VMHostName");
            this.userCreateVM_image = getTranslation("userCreateVM", "image");
            this.userCreateVM_nbCPU = getTranslation("userCreateVM", "nbCPU");
            this.userCreateVM_memoryCapacity = getTranslation("userCreateVM", "memoryCapacity");
            this.userCreateVM_diskSize = getTranslation("userCreateVM", "diskSize");
            this.userCreateVM_location = getTranslation("userCreateVM", "location");
            this.userCreateVM_btnChangeImage = getTranslation("userCreateVM", "btnChangeImage");
            this.userCreateVM_btnCreate = getTranslation("userCreateVM", "btnCreate");
            this.userCreateVM_btnCancel = getTranslation("userCreateVM", "btnCancel");
            this.userCreateVM_msgVMLabelMandatory = getTranslation("userCreateVM", "msgVMLabelMandatory");
            this.userCreateVM_msgVMLabelInvalid = getTranslation("userCreateVM", "msgVMLabelInvalid");
            this.userCreateVM_msgVMImageMandatory = getTranslation("userCreateVM", "msgVMImageMandatory");
            this.vmImageBrowser_title = getTranslation("vmImageBrowser", "title");
            this.vmImageBrowser_category = getTranslation("vmImageBrowser", "category");
            this.vmImageBrowser_hypervisor = getTranslation("vmImageBrowser", "hypervisor");
            this.vmImageBrowser_myImages = getTranslation("vmImageBrowser", "myImages");
            this.vmImageBrowser_publicImages = getTranslation("vmImageBrowser", "publicImages");
            this.vmImageBrowser_colCategory = getTranslation("vmImageBrowser", "colCategory");
            this.vmImageBrowser_colName = getTranslation("vmImageBrowser", "colName");
            this.vmImageBrowser_colDescription = getTranslation("vmImageBrowser", "colDescription");
            this.vmImageBrowser_colHypervisor = getTranslation("vmImageBrowser", "colHypervisor");
            this.vmImageBrowser_colArchitecture = getTranslation("vmImageBrowser", "colArchitecture");
            this.vmImageBrowser_colDiskSizeMB = getTranslation("vmImageBrowser", "colDiskSizeMB");
            this.commonVMImages_titleUser = getTranslation("commonVMImages", "titleUser");
            this.commonVMImages_titleAdmin = getTranslation("commonVMImages", "titleAdmin");
            this.commonVMImages_btnDestroyVMImages = getTranslation("commonVMImages", "btnDestroyVMImages");
            this.commonVMImages_destroyVMImagesConfirm = getTranslation("commonVMImages", "destroyVMImagesConfirm");
            this.commonVMImages_userFilter = getTranslation("commonVMImages", "userFilter");
            this.commonVMImages_categoryFilter = getTranslation("commonVMImages", "categoryFilter");
            this.commonVMImages_hypervisorFilter = getTranslation("commonVMImages", "hypervisorFilter");
            this.commonVMImages_colId = getTranslation("commonVMImages", "colId");
            this.commonVMImages_colStatus = getTranslation("commonVMImages", "colStatus");
            this.commonVMImages_colUser = getTranslation("commonVMImages", "colUser");
            this.commonVMImages_colCategory = getTranslation("commonVMImages", "colCategory");
            this.commonVMImages_colName = getTranslation("commonVMImages", "colName");
            this.commonVMImages_colDescription = getTranslation("commonVMImages", "colDescription");
            this.commonVMImages_colHypervisor = getTranslation("commonVMImages", "colHypervisor");
            this.commonVMImages_colArchitecture = getTranslation("commonVMImages", "colArchitecture");
            this.commonVMImages_colDiskSizeMB = getTranslation("commonVMImages", "colDiskSizeMB");
            this.userView_title = getTranslation("userView", "title");
            this.userView_infos = getTranslation("userView", "infos");
            this.userView_lastName = getTranslation("userView", "lastName");
            this.userView_firstName = getTranslation("userView", "firstName");
            this.userView_username = getTranslation("userView", "username");
            this.userView_email = getTranslation("userView", "email");
            this.userView_quotas = getTranslation("userView", "quotas");
            this.userView_free = getTranslation("userView", "free");
            this.userView_used = getTranslation("userView", "used");
            this.userView_cpuQuota = getTranslation("userView", "cpuQuota");
            this.userView_ramQuota = getTranslation("userView", "ramQuota");
            this.userView_diskQuota = getTranslation("userView", "diskQuota");
            this.userView_nbVMQuota = getTranslation("userView", "nbVMQuota");
            this.userView_cpuConsumption = getTranslation("userView", "cpuConsumption");
            this.userView_ramConsumption = getTranslation("userView", "ramConsumption");
            this.userView_diskConsumption = getTranslation("userView", "diskConsumption");
            this.userView_nbVMConsumption = getTranslation("userView", "nbVMConsumption");
            this.userView_SSHKey = getTranslation("userView", "SSHKey");
            this.userView_currentSSHKey = getTranslation("userView", "currentSSHKey");
            this.userView_newSSHKey = getTranslation("userView", "newSSHKey");
            this.userView_updateSSHKey = getTranslation("userView", "updateSSHKey");
            this.userView_passwords = getTranslation("userView", "passwords");
            this.userView_updateWebPassword = getTranslation("userView", "updateWebPassword");
            this.userView_webPassword = getTranslation("userView", "webPassword");
            this.userView_newWebPassword = getTranslation("userView", "newWebPassword");
            this.userView_newWebPassword2 = getTranslation("userView", "newWebPassword2");
            this.userView_updateAPIPassword = getTranslation("userView", "updateAPIPassword");
            this.userView_APIPassword = getTranslation("userView", "APIPassword");
            this.userView_newAPIPassword = getTranslation("userView", "newAPIPassword");
            this.userView_newAPIPassword2 = getTranslation("userView", "newAPIPassword2");
            this.userView_newPasswordError = getTranslation("userView", "newPasswordError");
            this.userView_oldPasswordError = getTranslation("userView", "oldPasswordError");
            this.inventory_title = getTranslation("inventory", "title");
            this.inventory_btnListOnly = getTranslation("inventory", "btnListOnly");
            this.inventory_btnSplit = getTranslation("inventory", "btnSplit");
            this.inventory_btnDetailOnly = getTranslation("inventory", "btnDetailOnly");
            this.inventory_colUser = getTranslation("inventory", "colUser");
            this.inventory_colVMName = getTranslation("inventory", "colVMName");
            this.inventory_colStatus = getTranslation("inventory", "colStatus");
            this.inventory_colIPAddress = getTranslation("inventory", "colIPAddress");
            this.inventory_colDns = getTranslation("inventory", "colDns");
            this.inventory_colImage = getTranslation("inventory", "colImage");
            this.inventory_colMemory = getTranslation("inventory", "colMemory");
            this.inventory_colDiskSize = getTranslation("inventory", "colDiskSize");
            this.inventory_colNbCpu = getTranslation("inventory", "colNbCpu");
            this.inventory_colCpuLoad = getTranslation("inventory", "colCpuLoad");
            this.inventory_colMemoryUsed = getTranslation("inventory", "colMemoryUsed");
            this.inventory_btnDestroyVM = getTranslation("inventory", "btnDestroyVM");
            this.inventory_destroyVMConfirm = getTranslation("inventory", "destroyVMConfirm");
            this.inventory_btnStartVM = getTranslation("inventory", "btnStartVM");
            this.inventory_btnStopVM = getTranslation("inventory", "btnStopVM");
            this.inventory_btnRebootVM = getTranslation("inventory", "btnRebootVM");
            this.inventory_btnPauseVM = getTranslation("inventory", "btnPauseVM");
            this.inventory_btnUnpauseVM = getTranslation("inventory", "btnUnpauseVM");
            this.inventory_btnConsoleVM = getTranslation("inventory", "btnConsoleVM");
            this.inventory_user = getTranslation("inventory", "user");
            this.inventory_dataCenter = getTranslation("inventory", "dataCenter");
            this.inventory_domain = getTranslation("inventory", "domain");
            this.inventory_serverPool = getTranslation("inventory", "serverPool");
            this.inventory_host = getTranslation("inventory", "host");
            this.inventory_virtualMachine = getTranslation("inventory", "virtualMachine");
            this.inventoryDomain_domain = getTranslation("inventoryDomain", "domain");
            this.inventoryServerPool_serverPool = getTranslation("inventoryServerPool", "serverPool");
            this.inventoryHost_host = getTranslation("inventoryHost", "host");
            this.inventoryVirtualMachine_virtualMachine = getTranslation("inventoryVirtualMachine", "virtualMachine");
            this.inventoryVirtualMachine_tabCpuScheduling = getTranslation("inventoryVirtualMachine", "tabCpuScheduling");
            this.inventoryVirtualMachine_tabNetworkInterfaces = getTranslation("inventoryVirtualMachine", "tabNetworkInterfaces");
            this.inventoryVirtualMachine_tabPerformances = getTranslation("inventoryVirtualMachine", "tabPerformances");
            this.inventoryVirtualMachine_tabConsole = getTranslation("inventoryVirtualMachine", "tabConsole");
            this.inventoryVirtualMachine_schedulingWeight = getTranslation("inventoryVirtualMachine", "schedulingWeight");
            this.inventoryVirtualMachine_schedulingCap = getTranslation("inventoryVirtualMachine", "schedulingCap");
            this.inventoryVirtualMachine_colDeviceName = getTranslation("inventoryVirtualMachine", "colDeviceName");
            this.inventoryVirtualMachine_colMacAddress = getTranslation("inventoryVirtualMachine", "colMacAddress");
            this.inventoryVirtualMachine_vncConsole = getTranslation("inventoryVirtualMachine", "vncConsole");
            this.inventoryVirtualMachine_msgConsoleNotAvailable = getTranslation("inventoryVirtualMachine", "msgConsoleNotAvailable");
            this.performances_cbPerfMetricsPrompt = getTranslation("performances", "cbPerfMetricsPrompt");
            this.performances_cbStacked = getTranslation("performances", "cbStacked");
            this.performances_cbPast30Seconds = getTranslation("performances", "cbPast30Seconds");
            this.performances_cbPastMinute = getTranslation("performances", "cbPastMinute");
            this.performances_cbPast5Minutes = getTranslation("performances", "cbPast5Minutes");
            this.performances_cbPast30Minutes = getTranslation("performances", "cbPast30Minutes");
            this.performances_cbPastHour = getTranslation("performances", "cbPastHour");
            this.performances_cbPastDay = getTranslation("performances", "cbPastDay");
            this.performances_cbPastMonth = getTranslation("performances", "cbPastMonth");
            this.performances_cbPastYear = getTranslation("performances", "cbPastYear");
            this.imageManagement_title = getTranslation("imageManagement", "title");
            this.userManagement_title = getTranslation("userManagement", "title");
            this.userManagement_btnDeleteUser = getTranslation("userManagement", "btnDeleteUser");
            this.userManagement_btnUpdateQuota = getTranslation("userManagement", "btnUpdateQuota");
            this.userManagement_btnCreateUser = getTranslation("userManagement", "btnCreateUser");
            this.userManagement_deleteUserConfirm = getTranslation("userManagement", "deleteUserConfirm");
            this.userManagement_colUsername = getTranslation("userManagement", "colUsername");
            this.userManagement_colFirstName = getTranslation("userManagement", "colFirstName");
            this.userManagement_colLastName = getTranslation("userManagement", "colLastName");
            this.userManagement_colEmail = getTranslation("userManagement", "colEmail");
            this.userManagement_colPortalPassword = getTranslation("userManagement", "colPortalPassword");
            this.userManagement_colApiPassword = getTranslation("userManagement", "colApiPassword");
            this.userManagement_updateQuota = getTranslation("userManagement", "updateQuota");
            this.userManagement_cpuQuota = getTranslation("userManagement", "cpuQuota");
            this.userManagement_storageQuota = getTranslation("userManagement", "storageQuota");
            this.userManagement_memoryQuota = getTranslation("userManagement", "memoryQuota");
            this.userManagement_vmQuota = getTranslation("userManagement", "vmQuota");
        }
        
        
        
        private function getTranslation(viewID : String, textID : String) : String
        {
            var translation : String = String(this.xmlTranslations.view.(@id == viewID).text.(@id == textID).attribute(this.locale));
            
            // Suppress Windows carriage return characters (\r)
            translation = translation.replace(/\r/g, '');
            
            return translation;
        }
    
    }
}


/**
 * Inner class used for the Singleton Design Pattern.
 * @author ARLERAY
 */
class SingletonEnforcer
{

}


