/**
 *
 * SIROCCO
 * Copyright (C) 2011 France Telecom
 * Contact: sirocco@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  $Id: Wizard.as 774 2011-12-18 16:25:10Z dangtran $
 *
 */

/*
Copyright 2009 Flashmattic, Matti Bar-Zeev

Licensed under the Apache License,Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software distributed
under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
CONDITIONS OF ANY KIND, either express or implied. See the License for the
specific language governing permissions and limitations under the License.
*/
package org.ow2.sirocco.cloudmanager.utils.wizard
{
    import org.ow2.sirocco.cloudmanager.model.CloudModelLocator;
    
    import flash.display.DisplayObject;
    import flash.events.MouseEvent;
    
    import mx.binding.utils.BindingUtils;
    import mx.binding.utils.ChangeWatcher;
    import mx.containers.BoxDirection;
    import mx.containers.ControlBar;
    import mx.containers.HBox;
    import mx.containers.TitleWindow;
    import mx.containers.VBox;
    import mx.containers.ViewStack;
    import mx.controls.Button;
    import mx.controls.HRule;
    import mx.controls.Text;
    import mx.controls.TextArea;
    import mx.core.UIComponent;
    import mx.core.mx_internal;
    import mx.events.CloseEvent;
    import mx.events.FlexEvent;
    
    use namespace mx_internal;
    
    
    /**
     *  Dispatched when the wizard is done
     *
     *  @eventType 	com.flashmattic.component.wizard.WizardEvent.WIZARD_COMPLETE
     *  @tiptext 	wizard complete event
     */
    [Event(name="wizardComplete", type="com.flashmattic.flashmatticomponents.framework.wizard.WizardEvent")]
    
    /**
     *  Dispatched when the wizard is canceled
     *
     *  @eventType 	com.flashmattic.component.wizard.WizardEvent.WIZARD_CANCEL
     *  @tiptext 	wizard complete event
     */
    [Event(name="wizardCancel", type="com.flashmattic.flashmatticomponents.framework.wizard.WizardEvent")]
    
    /**
     *  Dispatched when the wizard has moved to the next step
     *
     *  @eventType 	com.flashmattic.component.wizard.WizardEvent.NEXT_STEP
     *  @tiptext 	wizard complete event
     */
    [Event(name="nextStep", type="com.flashmattic.flashmatticomponents.framework.wizard.WizardEvent")]
    
    /**
     *  Dispatched when the wizard has moved to the previous step
     *
     *  @eventType 	com.flashmattic.component.wizard.WizardEvent.PREV_STEP
     *  @tiptext 	wizard complete event
     */
    [Event(name="prevStep", type="com.flashmattic.flashmatticomponents.framework.wizard.WizardEvent")]
    
    /**
     * @author MBz
     */
    public class Wizard extends TitleWindow
    {
        private const WIZARD_MIN_WIDTH : Number = 350;
        
        private const WIZARD_MIN_HEIGHT : Number = 250;
        
        private var _previousButton : Button;
        
        private var _nextButton : Button;
        
        private var _cancelButton : Button;
        
        private var _dataModel : *;
        
        private var _steps : Array;
        
        private var _stepDescriptionsArray : Array;
        
        private var _stepDescriptionText : Text;
        
        private var _separator : HRule;
        
        private var _stepsViewStack : ViewStack;
        
        private var _currentStepIndex : int;
        
        private var _stepsMaxWidth : Number = WIZARD_MIN_WIDTH;
        
        private var _stepsMaxHeight : Number = WIZARD_MIN_HEIGHT;
        
        private var _stepValidationWatcher : ChangeWatcher;
        
        private var _currentStep : IWizardStep;
        
        private var _showSummaryStep : Boolean;
        
        private var _summaryStep : IWizardStep;
        
        private var _summaryText : TextArea;
        
        private var _summarArray : Array;
        
        private var _summaryString : String;
        
        // Commit properties flags
        private var _wizardStepsChanged : Boolean = false;
        
        private var _hbox : HBox;
        
        private var _leftBox : VBox;
        
        private var _rightBox : VBox;
        
        private var _wizardStepList : WizardStepList;
        
        private var _stepsButtonArray : Array;
        
        private var modelLocator : CloudModelLocator = CloudModelLocator.getInstance();
        
        
        
        public function Wizard()
        {
            super();
            // This component has a closing button always. This button does what
            // canceling the whole wizard do.			
            //showCloseButton = true;
            addEventListener(CloseEvent.CLOSE, onClose);
            // Reset the current step to be the first
            currentStepIndex = 0;
            // Set the minimum scale of this wizard
            minWidth = WIZARD_MIN_WIDTH;
            minHeight = WIZARD_MIN_HEIGHT;
            // Reset the _summarArray 
            _summarArray = new Array();
            // Create the summary step although we're not sure we are actually 
            // going to use it
            createSummaryStep();
        }
        
        
        
        // OVERRIDES ------------------------------------------------------------
        override protected function createChildren() : void
        {
            super.createChildren();
            setStyle("paddingTop", 0);
            setStyle("paddingBottom", 0);
            setStyle("paddingLeft", 0);
//			setStyle("paddingRight",0);
            _hbox = new HBox();
            _hbox.percentHeight = 100;
            _hbox.percentWidth = 100;
            
            addChild(_hbox);
            _leftBox = new VBox();
            _leftBox.percentHeight = 100;
            initWizardStepsList();
            _hbox.addChild(_leftBox);
            _rightBox = new VBox();
            _rightBox.percentHeight = 100;
            _rightBox.percentWidth = 100;
            _hbox.addChild(_rightBox);
            // Create the control bar for the wizard
            initWizardControlBar();
            
            // Create the step description text control
            if (_stepDescriptionText == null)
            {
                _stepDescriptionText = new Text();
                _stepDescriptionText.percentWidth = 125;
                _stepDescriptionText.setStyle("fontWeight", "bold");
                _stepDescriptionText.setStyle("fontSize", 12);
                _stepDescriptionText.text = "Step description here...";
                _rightBox.addChild(_stepDescriptionText);
                _separator = new HRule();
                _separator.percentWidth = 100;
                _rightBox.addChild(_separator);
            }
            
            // Create the view stack to hold all the steps content
            if (_stepsViewStack == null)
            {
                _stepsViewStack = new ViewStack();
                _stepsViewStack.percentWidth = 100;
                _stepsViewStack.percentHeight = 100;
                _stepsViewStack.creationPolicy = "all";
                _stepsViewStack.addEventListener(FlexEvent.CREATION_COMPLETE, onStepsViewStackCreationComplete, false, 0, true);
                _rightBox.addChild(_stepsViewStack);
            }
        }
        
        
        
        override protected function commitProperties() : void
        {
            super.commitProperties();
            
            if (_wizardStepsChanged)
            {
                initWizardSteps();
                _wizardStepsChanged = false;
            }
        }
        
        
        
        /**
         * The Wizard sets it's own scale according to its steps width and
         * height. This is why we don't want the end developer to set the
         * scale, since it will only tampre with the auto scaling.
         * @param value
         */
        override public function set width(value : Number) : void
        {
            throw new Error("Width property cannot be set for the Wizard");
        }
        
        
        
        /**
         * The Wizard sets it's own scale according to its steps width and
         * height. This is why we don't want the end developer to set the
         * scale, since it will only tampre with the auto scaling.
         * @param value
         */
        override public function set height(value : Number) : void
        {
            throw new Error("Height property cannot be set for the Wizard");
        }
        
        
        
        // PRIVATES ------------------------------------------------------------
        /**
         * Lays down the buttons for "next", "previous" and "cancel". Also
         * attach event listeners for them.
         */
        private function initWizardControlBar() : void
        {
            // if controlBar doesn't exist, create the controlBar
            if (controlBar == null)
            {
                controlBar = new ControlBar();
                //add control bar
                addChildAt(controlBar as ControlBar, numChildren);
                ControlBar(controlBar).direction = BoxDirection.HORIZONTAL;
                ControlBar(controlBar).setStyle("horizontalAlign", "right");
                createComponentsFromDescriptors();
            }
            
            if (_previousButton == null)
            {
                _previousButton = new Button();
                _previousButton.addEventListener(MouseEvent.CLICK, onPrevious, false, 0, true);
                _previousButton.label = this.modelLocator.resourceManager.common_previous;
                ControlBar(controlBar).addChild(_previousButton);
            }
            
            // if the button doesn't exist, create button
            if (_nextButton == null)
            {
                _nextButton = new Button();
                _nextButton.label = this.modelLocator.resourceManager.common_next;
                _nextButton.addEventListener(MouseEvent.CLICK, onNext, false, 0, true);
                ControlBar(controlBar).addChild(_nextButton);
            }
            
            // if the button doesn't exist, create button
            if (_cancelButton == null)
            {
                _cancelButton = new Button();
                _cancelButton.label = this.modelLocator.resourceManager.common_cancel;
                _cancelButton.addEventListener(MouseEvent.CLICK, onCancel, false, 0, true);
                ControlBar(controlBar).addChild(_cancelButton);
            }
        }
        
        
        
        private function initWizardStepsList() : void
        {
            _stepsButtonArray = new Array();
            var i : Number = 0;
            
            for each (var step : IWizardStep in steps)
            {
                _stepsButtonArray.push({label: step.stepDescription, index: i});
                i++;
            }
            _wizardStepList = new WizardStepList();
            _wizardStepList.dataProvider = _stepsButtonArray;
            _leftBox.addChild(_wizardStepList);
        }
        
        
        
        /**
         * Adds all the steps to the steps view stack, and also sets the array
         * of description
         */
        private function initWizardSteps() : void
        {
            _stepDescriptionsArray = new Array();
            
            for each (var step : IWizardStep in steps)
            {
                _stepsViewStack.addChild(step as DisplayObject);
                _stepDescriptionsArray.push(step.stepDescription);
            }
            validateWizardCurrentStep();
        }
        
        
        
        private function validateWizardCurrentStep() : void
        {
            // Set the selected child of the steps view stack to the current
            // step of the wizard
            _stepsViewStack.selectedIndex = currentStepIndex;
            _wizardStepList.selectedIndex = currentStepIndex;
            // Set the description of the current step
            _stepDescriptionText.text = _stepDescriptionsArray[currentStepIndex];
            // Bind the validation of the step with the next step availability
            _currentStep = _stepsViewStack.selectedChild as IWizardStep;
            // Validate the next and previous step
            validateNextStep();
            validatePrevStep();
            
            // Setup the watcher for the next button
            if (_stepValidationWatcher)
            {
                _stepValidationWatcher.unwatch();
            }
            _stepValidationWatcher = BindingUtils.bindProperty(_nextButton, "enabled", _currentStep, "isValid");
        }
        
        
        
        /**
         * Wraps up the wizard by dispatching a WizardEvent.WIZARD_COMPLETE and
         * disabling all the buttons
         * event
         */
        private function wrapUpWizard() : void
        {
            _previousButton.enabled = false;
            _nextButton.enabled = false;
            _cancelButton.enabled = false;
            // Dispatch the WizardEvent.WIZARD_COMPLETE event
            var e : WizardEvent = new WizardEvent(WizardEvent.WIZARD_COMPLETE);
            dispatchEvent(e);
        }
        
        
        
        /**
         * When the view stack containing all the steps has finished creating
         * itself, and it created them all since we've ordered it to do wo, we
         * now wish the get the max width and height of the the steps and set it
         * to the voew stack. we do that so that the end developer using this
         * component won't need to care about sizing his Wizard, and all the
         * scaling will be done behind the scenes.
         * @param event
         */
        private function scaleStepsViewStack() : void
        {
            for (var i : int = 0; i < _stepsViewStack.numChildren; i++)
            {
                var step : UIComponent = _stepsViewStack.getChildAt(i) as UIComponent;
                
                if (step.explicitWidth > _stepsMaxWidth)
                {
                    _stepsMaxWidth = step.explicitWidth;
                }
                
                if (step.measuredWidth > _stepsMaxWidth)
                {
                    _stepsMaxWidth = step.measuredWidth;
                }
                
                if (step.explicitHeight > _stepsMaxHeight)
                {
                    _stepsMaxHeight = step.explicitHeight;
                }
                
                if (step.measuredHeight > _stepsMaxHeight)
                {
                    _stepsMaxHeight = step.measuredHeight;
                }
            }
            _stepsViewStack.width = _stepsMaxWidth
            _stepsViewStack.height = _stepsMaxHeight;
            _stepsViewStack.invalidateSize();
        }
        
        
        
        /**
         * Get the step desicion from the current step and store it in the data
         * model
         */
        private function retreiveCurrentStepDecision() : void
        {
            if (_currentStep.dataField)
            {
                dataModel[_currentStep.dataField] = _currentStep.stepDecision;
                _summarArray[_currentStep.stepName] = _currentStep.readableStepDecision;
            }
        }
        
        
        
        /**
         * Gather the information from the _summaryDict and set it in a nice
         * string format to be displayed later on the summary step
         */
        private function setSummaryContent() : void
        {
            _summaryString = "";
            
            for (var stepName : String in _summarArray)
            {
                if (stepName != "null")
                {
                    _summaryString += "<B>" + stepName + ":</B><BR>" + _summarArray[stepName] + "<BR><BR>";
                }
            }
            // Set the summary string to the summary step text
            _summaryText.htmlText = _summaryString;
        }
        
        
        
        /**
         * When there are still steps ahead we keep the "Next" label to the
         * next button, but when we reach the final step, it's time to put the
         * "Finish" label on it
         */
        private function validateNextStep() : void
        {
            var nextStepIndex : int = currentStepIndex + 1;
            
            if (nextStepIndex >= steps.length)
            {
                _nextButton.label = this.modelLocator.resourceManager.common_finish;
                // Set the summary string to the summary step text
                setSummaryContent();
                
                if (this.modelLocator.TARGET_AUDIENCE == "DRI" && this.modelLocator.userLdapVo != null && this.modelLocator.userLdapVo.email == 'Unknown user')
                {
                    _nextButton.visible = false;
                }
                
            }
            else
            {
                _nextButton.label = this.modelLocator.resourceManager.common_next;
            }
        }
        
        
        
        /**
         * Sets the previous button state according to the current step index.
         * In plain words, if we don't have anywhere to go backwards, simply
         * remove the previous button.
         */
        private function validatePrevStep() : void
        {
            if (_previousButton)
            {
                _previousButton.visible = _previousButton.enabled = currentStepIndex > 0;
                
                if ((this.modelLocator.TARGET_AUDIENCE == "DRI" && this.modelLocator.userLdapVo != null && this.modelLocator.userLdapVo.email != 'Unknown user') || currentStepIndex == 1)
                {
                    _nextButton.visible = true;
                }
            }
        }
        
        
        
        /**
         * Creates the summary step and in it creates the TextArea that will
         * hold the summary information for the wizard. At this point we don't
         * even know if we're going to use the summary step but we create it
         * anyway since the "code-smell" of many IF's is more disturbing than
         * the performance price we pay.
         */
        private function createSummaryStep() : void
        {
            _summaryStep = new WizardStep();
            _summaryStep.isValid = true;
            _summaryStep.stepDescription = this.modelLocator.resourceManager.common_summary;
            _summaryStep.percentHeight = 100;
            _summaryStep.percentWidth = 100;
            _summaryText = new TextArea();
            _summaryText.percentHeight = 100;
            _summaryText.percentWidth = 100;
            _summaryStep.addChild(_summaryText);
        }
        
        
        
        // EVENT HANDLERS ------------------------------------------------------
        private function onStepsViewStackCreationComplete(event : FlexEvent) : void
        {
            scaleStepsViewStack();
        }
        
        
        
        /**
         * Called when the user has pressed the close button on the wizard
         * @param event
         */
        private function onClose(event : CloseEvent) : void
        {
            cancelWizard();
        }
        
        
        
        /**
         * Called when the user uses the cancel button on the wizard
         * @param event
         */
        private function onCancel(event : MouseEvent) : void
        {
            cancelWizard();
        }
        
        
        
        private function onPrevious(event : MouseEvent) : void
        {
            currentStepIndex--;
            validateWizardCurrentStep();
            // Dispatch the WizardEvent.PREV_STEP event
            var e : WizardEvent = new WizardEvent(WizardEvent.PREV_STEP);
            dispatchEvent(e);
        }
        
        
        
        private function onNext(event : MouseEvent) : void
        {
            // Get the step desicion from the current step
            retreiveCurrentStepDecision();
            
            // Go to the next step
            // If there are no more steps in the wizard, wrap it up
            if (++currentStepIndex == _steps.length)
            {
                wrapUpWizard();
            }
            else
            {
                validateWizardCurrentStep();
                // Dispatch the WizardEvent.NEXT_STEP event
                var e : WizardEvent = new WizardEvent(WizardEvent.NEXT_STEP);
                dispatchEvent(e);
            }
        }
        
        
        
        /**
         * A routine of action to be executed when the user decided to cancel
         * the wizard
         * - Dispatching an event to indicate the cancelation
         */
        private function cancelWizard() : void
        {
            var e : WizardEvent = new WizardEvent(WizardEvent.WIZARD_CANCEL);
            dispatchEvent(e);
        }
        
        
        
        // GETTER/SETTER -------------------------------------------------------
        public function get dataModel() : *
        {
            return _dataModel;
        }
        
        
        
        public function set dataModel(value : *) : void
        {
            _dataModel = value;
        }
        
        
        
        public function get steps() : Array
        {
            return _steps;
        }
        ;
        
        
        
        public function set steps(value : Array) : void
        {
            _steps = value;
            
            if (_showSummaryStep)
            {
                _steps.push(_summaryStep);
            }
            _wizardStepsChanged = true;
            invalidateProperties();
        }
        
        
        
        private function set currentStepIndex(value : int) : void
        {
            _currentStepIndex = value;
        }
        
        
        
        private function get currentStepIndex() : int
        {
            return _currentStepIndex;
        }
        
        
        
        public function set showSummaryStep(value : Boolean) : void
        {
            _showSummaryStep = value;
        }
    }
}


